#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Get fastq files and prepare them for other purposes. 
"""

## import useful modules
import time
import os
import sys
from io import open
import shutil
import pandas as pd
from termcolor import colored

## import my modules
from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

################################
def run_prep(options):
	"""
	Main function of the prep module.
	
	This module prepares fastq files for later usage. It initially checks the length
	of the name and advises the user to rename samples if exceeded. Along ``BacterialTyper`` 
	there are a few string length limitations by different software that need to be sort
	out from the beginning of the process.
	
	This module allows to user to copy files into the project folder initiate or only link using
	a symbolic link to avoid duplicated raw data. 
	
	See additional details of this module in user_guide :ref:`prep module entry<prep-description>`. 

	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.sampleParser.help_format`
		
		- :func:`BacterialTyper.scripts.sampleParser.get_files`
		
		- :func:`BacterialTyper.scripts.sampleParser.one_file_per_sample`
	
		- :func:`BacterialTyper.scripts.functions.pipeline_header`
		
		- :func:`BacterialTyper.scripts.functions.boxymcboxface`
	
		- :func:`BacterialTyper.scripts.functions.print_time`
		
		- :func:`BacterialTyper.scripts.functions.create_folder`
		
		- :func:`BacterialTyper.scripts.functions.outdir_project`
		
		- :func:`BacterialTyper.scripts.functions.create_subfolder`
		
		- :func:`BacterialTyper.scripts.functions.get_symbolic_link`
		
		- :func:`BacterialTyper.scripts.functions.create_human_timestamp`

	"""
	
	## help_format option
	if (options.help_format):
		sampleParser.help_fastq_format()
		exit()
		
	functions.pipeline_header()
	functions.boxymcboxface("Preparing samples")
	print ("--------- Starting Process ---------")
	functions.print_time()
	
	## init time
	start_time_total = time.time()
	
	## absolute path for in & out
	input_dir = os.path.abspath(options.input)
	outdir = os.path.abspath(options.output_folder)

	### set as default paired_end mode
	if (options.single_end):
		options.pair = False
	else:
		options.pair = True

	## Project mode as default
	project_mode=True
	if (options.detached):
		options.project = False
		project_mode=False
	else:
		options.project = True

	## output folder	
	print ("\n+ Create output folder(s):")
	functions.create_folder(outdir)

	### info
	final_dir = ""
	if (options.project):
		print ("+ Generate a directory containing information within the project folder provided")
		final_dir = functions.create_subfolder("info", outdir)
	else:
		final_dir = outdir
	
	## get files
	pd_samples_retrieved = sampleParser.get_files(options, input_dir, "fastq", ("fastq", "fq", "fastq.gz", "fq.gz"))
		
	## Information returned in pd_samples_retrieved
	### sample, dirname, name, name_len, lane, read_pair, lane_file, ext, gz
	
	if options.debug:
		print (colored("** DEBUG: pd_samples_retrieved", 'yellow'))
		functions.print_all_pandaDF(pd_samples_retrieved)
	
	## time stamp
	start_time_partial = functions.timestamp(start_time_total)
	
	## check character limitation
	list_lengths = pd_samples_retrieved.loc[:,'name_len'].to_list()
	if any(i > 10 for i in list_lengths):
		print (colored("\t ** Name lengths exceeds the 10 character limitation...", 'yellow'))
		if not (options.rename):
			print (colored("** ERROR: Rename files or provide --rename option...", 'red'))
			exit()

	### rename files 
	if (options.rename):
		options.rename = os.path.abspath(options.rename)
		if not functions.is_non_zero_file(options.rename):
			print (colored("** ERROR: File provided with rename information is not readable.", 'red'))
			print (options.rename)
			exit()
		
		names_retrieved = pd.read_csv(options.rename, sep=',', 
									index_col=0, squeeze=True, 
									header=None).to_dict() ## read csv to dictionary
		if (options.debug):
			print (colored('** DEBUG: names_retrieved', 'yellow'))
			print (names_retrieved)
			
		## TODO: check integrity of new names and special characters
	
		## print to a file
		timestamp = functions.create_human_timestamp()
		rename_details = final_dir + '/' + timestamp + '_prep_renameDetails.txt'
		rename_details_hd = open(rename_details, 'w')
	
		## rename files 		
		for index, row in pd_samples_retrieved.iterrows():
			if (row['gz']):
				extension_string = row['ext'] + row['gz']
			else:
				extension_string = row['ext']
			
			if options.single_end:
				renamed = names_retrieved[row['name']] + '.' + extension_string
			else:
				renamed = names_retrieved[row['name']] + '_' + row['read_pair'] + '.' + extension_string
			
			## modify frame
			pd_samples_retrieved.loc[index, 'new_file'] = renamed
			pd_samples_retrieved.loc[index, 'new_name'] = names_retrieved[row['name']]
			## save in file
			string = row['sample'] + '\t' + renamed + '\n'
			rename_details_hd.write(string)
			
			if (options.debug):
				print (colored('** DEBUG: rename', 'yellow'))
				print ("Original: ", row['name'])
				print ("Renamed: ", names_retrieved[row['name']])
				print ("File:", renamed)
		
		rename_details_hd.close()	

		##elif (options.single_end): It should work for both
		print ("+ Sample files have been renamed...")
	else:
		pd_samples_retrieved['new_file'] = pd_samples_retrieved['file']

	## create outdir for each sample
	outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "raw")	
		
	## merge option
	if (options.merge):
		print ("+ Sample files will be merged...")
		## TODO: check when rename option provided
		pd_samples_merged = sampleParser.one_file_per_sample(
			pd_samples_retrieved, outdir_dict, options.threads,	
			final_dir, options.debug)
		
		if (options.rename):
			print ("+ Merge files have been renamed...")
		else:
			print ("+ Sample files have been merged...")
		
		## process is finished here
		print ("\n*************** Finish *******************")
		start_time_partial = functions.timestamp(start_time_total)
	
		print ("+ Exiting prep module.")
		exit()
	
	## debugging messages
	if (options.debug):
		print (colored("** DEBUG: pd_samples_retrieved", 'yellow'))
		functions.print_all_pandaDF(pd_samples_retrieved)
		print (colored("** DEBUG: outdir_dict", 'yellow'))
		print (outdir_dict)
	
	## copy or create symbolic link for files
	if (options.copy):
		print ("+ Sample files will be copied...")
		## print to a file
		timestamp = functions.create_human_timestamp()
		copy_details = final_dir + '/' + timestamp + '_prep_copyDetails.txt'
		copy_details_hd = open(copy_details, 'w')
	else:
		print ("+ Sample files will be linked...")	
	
	list_reads = []
	for index, row in pd_samples_retrieved.iterrows():
		if (options.copy):
		    ## TODO: debug & set threads to copy faster
		    shutil.copy(row['sample'], os.path.join(outdir_dict[row['new_name']], row['new_file'] ))            
		    string = row['sample'] + '\t' + os.path.join(outdir_dict[row['new_name']], row['new_file']) + '\n'
		    copy_details_hd.write(string)            
		else:
		    list_reads.append(row['new_file'])
		    
		    if options.project:
		        functions.get_symbolic_link_file(row['sample'], 
		                                         os.path.join(outdir_dict[row['new_name']], row['new_file']))

	if (options.copy):
		print ("+ Sample files have been copied...")
		copy_details_hd.close()
	else:
		if not options.project:
			functions.get_symbolic_link(list_reads, outdir)
	
	print ("\n*************** Finish *******************")
	start_time_partial = functions.timestamp(start_time_total)

	print ("+ Exiting prep module.")
	return()


#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                        ##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain        ##
##########################################################
"""
Creates a report in HTML for each step of the process analyzed.
"""
## import useful modules
import os
import sys
import re
import time
from termcolor import colored
import pandas as pd
from Bio import SeqIO

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.scripts import database_user
from BacterialTyper.config import set_config
from BacterialTyper.report import retrieve_genes
from BacterialTyper.report.Staphylococcus import get_spa_typing

## example report check: https://github.com/tseemann/nullarbor

## for staphyloccocus aureus: https://genomemedicine.biomedcentral.com/articles/10.1186/s13073-018-0593-7
## for klebsiella: kleborate
## for micobacterium tuberculosis: MTBSeq

## R package: http://bioconductor.org/packages/release/bioc/html/ReportingTools.html

####################################
def run_report(options):
    
    ## init time
    start_time_total = time.time()
    
    ##################################
    ### show help messages if desired    
    ##################################

    if (options.help_spaTyper):
        ## help_format option
        get_spa_typing.help_spaTyper()
        exit()

    elif (options.help_project):
        ## information for project
        help_info.project_help()
        exit()


    ## debugging messages
    global Debug
    if (options.debug):
        Debug = True
    else:
        Debug = False

    ### set as default paired_end mode
    if (options.single_end):
        options.pair = False
    else:
        options.pair = True

    ## message header
    functions.pipeline_header()
    functions.boxymcboxface("Report generation module")
    print ("--------- Starting Process ---------")
    functions.print_time()
    
    ## call assemble using spades
    start_time_partial = start_time_total
    
    ## absolute path for in & out
    options.database = os.path.abspath(options.database)
    global input_dir
    input_dir = os.path.abspath(options.input)
    outdir=""

    ## set mode: project/detached
    global Project
    if (options.detached):
        options.project = False
        outdir = os.path.abspath(options.output_folder)
        Project=False
    else:
        options.project = True
        outdir = input_dir    
        Project=True

    ##
    print ("\n+ Get project information:")
    
    ## get files: trimm, assembly, annotation
    pd_samples_retrieved = database_user.get_userData_files(options, input_dir)
    
    ## get info: profile, ident, cluster, MGE
    pd_samples_info = database_user.get_userData_info(options, input_dir)
    
    ## get databases to list
    #retrieve_databases = get_options_db(options)

    ## debug message
    if (Debug):
        print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
        print (pd_samples_retrieved)
        
        print (colored("**DEBUG: pd_samples_info **", 'yellow'))
        print (pd_samples_info)
        
    ## generate output folder, if necessary
    print ("\n\n\n+ Generate a report summarizing analysis and sample information")
    if not options.project:
        functions.create_folder(outdir)
        outdir_report = outdir
    else:
        ### report generation
        outdir_report = functions.create_subfolder("report", outdir)
    
    ## create report with all data
    summary_report = functions.create_subfolder("summary_report", outdir_report)
    print ("Folder: ", summary_report)
    
    ## time stamp
    start_time_partial = functions.timestamp(start_time_partial)
    
    ########################################
    ## create species specific report if any
    ########################################
    if (options.species_report):
        ## Saureus
        if options.species_report == "Saureus":
            Saureus_specific(pd_samples_retrieved, pd_samples_info, options, summary_report)
        
        ## else
        ## to add accordingly        
            
        ## time stamp
        start_time_partial = functions.timestamp(start_time_partial)
        
    ###########################################################
    ## create gene fasta sequences retrieval if desired
    ###########################################################
    if options.genes_ids_fasta:
        ## given a list of genes ids, retrieve sequence for all samples from profile 
        if os.path.isfile(os.path.abspath(options.genes_ids_fasta)):
            in_file = os.path.abspath(options.genes_ids_fasta)
            gene_names = [line.rstrip('\n') for line in open(in_file)]
            print ('+ Retrieve selected genes sequences from the profile analysis for each sample.')
            print ('+ Searching gene:')
            
            ## get profiles available
            results_geneIDs = pd.DataFrame(columns=('sample', 'gene', 'id', 'sequence'))
            sample_frame = pd_samples_info.groupby(["name"])
            for g in gene_names:
                print ("\t+", g)
                for name, cluster_df in sample_frame:
                    my_list_profiles = cluster_df.loc[cluster_df['tag'] == 'profile']['ext'].to_list()
                    if options.debug:
                        print ("name: ", name)
                        print ("my_list_profiles:")
                        print (my_list_profiles)
                    
                    for p in my_list_profiles:
                        main_profile_folder = cluster_df.loc[cluster_df['ext'] == p]['dirname'].to_list()[0]
                        p = p.lower()
                        if p == 'vfdb':
                            p = p + '_full'
                        
                        profile_folder = os.path.join(main_profile_folder, p)
                        (seq_id, seq_sequence) = retrieve_genes.retrieve_genes_ids_sequences(profile_folder, g, Debug)
                        if (seq_id):
                             ## save results 
                             results_geneIDs.loc[len(results_geneIDs)] = (name, g, seq_id, seq_sequence)

        ## save for each gene in a separate fasta file
        list_of_genes = set(results_geneIDs['gene'].to_list())

        ## debug
        if Debug:
            print ("** DEBUG **")
            print (results_geneIDs) 
            print (list_of_genes)
        
        ## Save results
        genes_folder = functions.create_subfolder('genes', summary_report)
        for gene_retrieved in list_of_genes:
            this_frame = results_geneIDs[results_geneIDs['gene'] == gene_retrieved]
        
            gene_retrieved_file = os.path.join(genes_folder, gene_retrieved)
            gene_retrieved_fasta = gene_retrieved_file + ".fasta"
            gene_retrieved_info = gene_retrieved_file + "_info.txt"
            fasta_hd = open(gene_retrieved_fasta, 'w')
            info_hd = open(gene_retrieved_info, 'w')
            
            for item, row in this_frame.iterrows():
                string2write = ">" + row['sample'] + '_' + row['gene'] + '\n' + row['sequence'] + '\n'  
                string2write_info = row['sample'] + '\t' + row['gene'] + '\t' + row['id'] + '\n'
                fasta_hd.write(string2write)
                info_hd.write(string2write_info)
                
            fasta_hd.close()
            info_hd.close()
        
        ## time stamp
        start_time_partial = functions.timestamp(start_time_partial)
    
    ########################################
    ## create gene specific report if any
    ########################################
    if options.genes_ids_profile:
        if options.species_report == "Saureus":
            if Debug:
                print ("** options.genes_ids_profile **")
                print ("Analysis already done for Saureus")
        else:
            in_file = os.path.abspath(options.genes_ids_profile)
            gene_names = [line.rstrip('\n') for line in open(in_file)]
            results_Profiles = retrieve_genes.get_genes_profile(pd_samples_info, gene_names, options.debug, "name")
            if options.debug:
                print ("results_Profiles")
                print (results_Profiles)
        
            
            ## open excel writer
            name_excel = summary_report + '/gene_ids_profile.xlsx'
            writer = pd.ExcelWriter(name_excel, engine='xlsxwriter')
            results_Profiles.to_excel(writer, sheet_name="gene_ids") 
            
            ## close
            writer.save()

            ## time stamp
            start_time_partial = functions.timestamp(start_time_partial)
        
    ###############################################
    ## Search for any additional fasta sequence
    ###############################################
    if options.genes_fasta:
        ## given a list of fasta sequences search using blast against proteins annotated or genome
        print("** THIS OPTION IS NOT IMPLEMENTED YET... **")
    
    print ("\n*************** Finish *******************")
    start_time_partial = functions.timestamp(start_time_total)

    print ("+ Exiting Report generation module.")
    return()

#######################3
def Saureus_specific(samples_df, samples_info, options, folder):
    """
    Retrieves Saureus specific information.
    
    See additional information in :doc:`../../user_guide/report/Saureus/saureus_report`
    """
    
    ########################################
    ## get European Quality Control genes
    ########################################
    Staphylococcus_path = os.path.abspath( os.path.join( os.path.realpath(__file__), '..', '..', 'report', 'Staphylococcus'))
    EQC_genes = os.path.join(Staphylococcus_path, "EQC_genes.csv")
    arcA_gene = os.path.join(Staphylococcus_path, "arcA.fasta")
    
    EQC_genes_df = functions.get_data(EQC_genes, ',', '')
    ## Gene,ID,Source
    ## mecA,ARO:3000617,CARD
    ## mecC,ARO:3001209,CARD
    ## mupA,ARO:3000521,CARD
    
    ## debugging messages
    if options.debug:
        print ("## DEBUG: Saureus_specific")
        print (Staphylococcus_path)
        print (arcA_gene)
        
        print ("EQC genes")
        print (EQC_genes)
        print (EQC_genes_df)
 

    ####################
    ## get gene info by unique ID
    ####################
    ## get gene names
    gene_IDs = EQC_genes_df['ID'].to_list()
    
    results_Profiles_ids = retrieve_genes.get_genes_profile(samples_info, gene_IDs, options.debug, 'ID')
    if options.debug:
        print ("results_Profiles")
        print (results_Profiles)
    
    ########################################
    ## add additional genes if required
    ########################################
    if options.genes_ids_profile:
        in_file = os.path.abspath(options.genes_ids_profile)
        gene_names = [line.rstrip('\n') for line in open(in_file)]
        
        if options.debug:
            print ("gene_names")
            print (gene_names)
    
        results_Profiles_names = retrieve_genes.get_genes_profile(samples_info, gene_names, options.debug, 'name')
        if options.debug:
            print ("results_Profiles")
            print (results_Profiles)
        
            
    #################################
    ## get blast sequence         ###
    #################################
    # arcA_gene
    
    ####################
    ## get spatyping  ##
    ####################
    samples_df = samples_df.set_index('name')
    assembly_files = samples_df.loc[samples_df['tag'] == "assembly", "sample"]
    results_spaType = pd.DataFrame()
    results_spaType = get_spa_typing.module_call(options.database, assembly_files.to_dict(), options.debug)
    
    ####################
    ## get sccmec
    ####################
    ## todo
    
    ####################
    ## save results
    ####################
    ## open excel writer
    name_excel = folder + '/Saureus_report.xlsx'
    writer = pd.ExcelWriter(name_excel, engine='xlsxwriter')
    
    # results_Profiles ids
    results_Profiles_ids.to_excel(writer, sheet_name="gene_ids")

    if options.genes_ids_profile:
        # results_Profiles names
        results_Profiles_ids.to_excel(writer, sheet_name="gene_names")

    # results_spaType
    results_spaType.to_excel(writer, sheet_name="spaTyper")

    ## close
    writer.save()




#!/usr/bin/env python3
#########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Sets configuration of the pipeline.

This module relies in several scripts and files which are arranged in the 
``BacterialTyper/config`` :doc:`folder <../config/config_index>`:	

	- :doc:`BacterialTyper/config/set_config.py <../config/set_config>`

	- :doc:`BacterialTyper/config/extern_progs.py <../config/extern_progs>`

	- :doc:`BacterialTyper/config/install_dependencies.py <../config/install_dependencies>`

	- Additional information and specifications :doc:`details <../config/additional_info>`.	

.. seealso:: Additional information on BacterialTyper configuration and requirements

	- :doc:`Installation <../../user_guide/installation/installing>`
	
	- :doc:`Requirements <../../user_guide/installation/requirements>` 

	- :doc:`Configuration <../../user_guide/modules/config>`

"""
## useful imports
import time
import io
import os
import sys
from termcolor import colored
from distutils.version import LooseVersion

## import my modules
from BacterialTyper.config import set_config
from BacterialTyper.config import extern_progs
from BacterialTyper.scripts import functions
from BacterialTyper.config import install_dependencies

def run(options):
	"""
	This is the main function of the module ``config``. It basically checks 
	if the different requirements (python` and third-party software) are
	fulfilled. 

	If any requirement is not available this modules tries to install them or reports to the user to
	manually install them.

	:param option: State whether to check or install missing modules, packages and third party software. Provide: check/install
	:param install_path: Absolute path to install modules or packages missing. Default: ``BacterialTyper`` environment path.
	:param IslandPath: True/False for checking additional perl and software required by this option analysis.
	:param debug: True/false for debugging messages.
	
	:type option: string 
	:type IslandPath: boolean
	:type install_path: string 
	:type debug: boolean	

	.. seealso:: This function depends on several ``BacterialTyper`` functions:

		- :func:`BacterialTyper.config.set_config.check_python_packages`

		- :func:`BacterialTyper.config.set_config.check_perl_packages`

		- :func:`BacterialTyper.config.extern_progs.return_min_version_soft`

		- :func:`BacterialTyper.config.extern_progs.print_dependencies`

	"""

	## init time
	start_time_total = time.time()

	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False

	functions.pipeline_header()
	functions.boxymcboxface("Pipeline Configuration")
	print ("--------- Starting Process ---------")
	functions.print_time()

	if (options.install_path):
		if os.path.isdir(options.install_path):
			if (Debug):
				print ("Installation path provided for missing modules, packages, dependencies...")
				print ("Path: " + options.install_path)
		else:
			print (colored("\n*** ERROR ****", 'red'))
			print (colored("Path provided is not a folder", 'red')) 
			print (options.install_path)
			exit()
	else:
		## get python environment path
		env_bin_directory = os.path.dirname(os.environ['_'])

		##os.path.abspath(os.path.join(os.path.dirname( __file__ ), '..', 'templates'))
		options.install_path = os.path.abspath(os.path.join(env_bin_directory, '../software'))

		if (Debug):
			print ("Retrieve environment path as installation path:")
			print ("Path: " + options.install_path)

		functions.create_folder(options.install_path)

	## install or only check
	option_install = False
	if (options.option == 'install'):
		print ("\n+ Check dependencies")
		print ("+ Try to install all missing dependencies, modules or third party software...")
		option_install = True

		## check if access and permission
		if os.path.isdir(options.install_path):
			if (set_config.access_check(options.install_path, mode=os.F_OK)):
				print ("Installation path is accessible and has permission for installation if necessary")
			else:
				print (colored("\n*** ERROR ****", 'red'))
				print (colored("No access/permission for this path: %s" %options.install_path, 'red'))
				print (colored("Please provide a valid path with access/permission to install any missing dependencies.", 'red'))
				exit()
		else:
			print (colored("\n*** ERROR ****", 'red'))
			print (colored("Path provided is not a folder", 'red'))
			print (options.install_path)
			exit()

	elif (options.option == 'only_check'):
		print ("\nCheck dependencies, modules or third party software and print report...")

	## python version
	functions.print_sepLine("+", 20, False)
	print ('Python:')
	functions.print_sepLine("+", 20, False)

	this_python_version = str(sys.version)
	python_min_version = extern_progs.return_min_version_soft('python')
	if LooseVersion(this_python_version) >= LooseVersion(python_min_version):
		print (colored("Minimum version (%s) satisfied: %s" %( python_min_version, this_python_version), 'green'))
	else:
		print (colored("Minimum version (%s) not satisfied: %s" %(python_min_version, this_python_version), 'red'))
		exit()
		
	## perl_version
	print ('\n')
	functions.print_sepLine("+", 50, False)
	print ('Perl:')
	functions.print_sepLine("+", 50, False)
	
	perl_min_version = extern_progs.return_min_version_soft('perl')
	this_perl_path = set_config.get_exe("perl", Debug)
	this_perl_version = set_config.get_version("perl", this_perl_path, Debug)
	if LooseVersion(this_perl_version) >= LooseVersion(perl_min_version):
		print (colored("Minimum version (%s) satisfied: %s" %(perl_min_version, this_perl_version), 'green'))
	else:
		print (colored("Minimum version (%s) not satisfied: %s" %(perl_min_version, this_perl_version), 'red'))
		exit()

	## third-party software
	print ('\n')
	functions.print_sepLine("+", 20, False)
	print ('External dependencies:')
	functions.print_sepLine("+", 20, False)
	
	set_config.check_dependencies(option_install, options.install_path, Debug)
	print ('\n')	

	## python packages
	print ('\n')
	functions.print_sepLine("+", 20, False)
	print ('Python packages:')
	functions.print_sepLine("+", 20, False)

	set_config.check_python_packages(Debug, option_install, options.install_path)
	functions.print_sepLine("+", 20, False)
	print ('\n')

	## perl packages
	print ('\n')
	functions.print_sepLine("+", 20, False)
	print ('Perl packages:')
	functions.print_sepLine("+", 20, False)

	set_config.check_perl_packages("perl_dependencies", Debug, option_install, options.install_path)
	functions.print_sepLine("+", 20, False)
	print ('\n')

	## IslandPath dependencies
	if (options.IslandPath):
		print ('\n')
		functions.print_sepLine("+", 20, False)
		print ('IslandPath packages and software required:')
		functions.print_sepLine("+", 20, False)
	
		set_config.check_IslandPath(Debug, option_install, options.install_path)
		functions.print_sepLine("+", 20, False)
		print ('\n')


	## R packages
	print ('\n')
	functions.print_sepLine("+", 20, False)
	print ('R packages:')
	functions.print_sepLine("+", 20, False)

	set_config.check_R_packages(option_install, options.install_path, Debug)
	functions.print_sepLine("+", 20, False)
	print ('\n')



#!/usr/bin/env python3
###########################################################
## Jose F. Sanchez										 ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain	 ##
###########################################################
"""
This module generates a uniq call of BacterialTyper to 
any other module. It automates the analysis and process.
"""
## useful imports
import time
import io
import os
import re
import sys
import concurrent.futures
from termcolor import colored
import pandas as pd

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

from BacterialTyper.modules import prep
from BacterialTyper.modules import qc
from BacterialTyper.modules import ident
from BacterialTyper.modules import profile
from BacterialTyper.modules import database
from BacterialTyper.modules import MGE
from BacterialTyper.modules import cluster
from BacterialTyper.modules import assemble
from BacterialTyper.modules import annot
from BacterialTyper.modules import database
from BacterialTyper.modules import trimm
#from BacterialTyper.modules import report_generation
from BacterialTyper.modules import help_info

from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import BUSCO_caller
from BacterialTyper.scripts import multiQC_report
from BacterialTyper.scripts import annotation
from BacterialTyper.scripts import ariba_caller
from BacterialTyper.scripts import min_hash_caller
from BacterialTyper.scripts import trimmomatic_call
from BacterialTyper.scripts import species_identification_KMA
from BacterialTyper.scripts import MLSTar
from BacterialTyper.scripts import bacteriophage


####################################
def run_BacterialTyper(options):

	## init time
	start_time_total = time.time()

	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False

	##################################
	### show help messages if desired	
	##################################
	if (options.help_format):
		## help_format option
		sampleParser.help_format()
		exit()

	elif (options.help_BUSCO):
		## information for BUSCO
		BUSCO_caller.print_help_BUSCO()
		exit()

	elif (options.help_project):
		## information for project
		help_info.project_help()
		exit()

	elif (options.help_multiqc):
		## information for Multiqc
		multiQC_report.multiqc_help()

	elif (options.help_Prokka):
		## information for Prokka
		annotation.print_list_prokka()
		exit()
	
	elif (options.help_Mash):
		## information for Min Hash Software
		min_hash_caller.helpMash()
		exit()
		
	elif (options.help_ARIBA):
		## information for ARIBA
		ariba_caller.help_ARIBA()
		exit()
        
	elif (options.help_trimm_adapters):
		## help on trimm adapters
		trimmomatic_call.print_help_adapters()
		exit()

	elif (options.help_KMA):
		## information for KMA Software
		species_identification_KMA.help_kma_database()
		exit()

	elif (options.help_MLSTar):
		## information for KMA Software
		MLSTar.help_MLSTar()
		exit()
	
	elif (options.help_PhiSpy):
		## information for PhiSpy software
		bacteriophage.help_PhiSpy()
		exit()

	elif (options.help_MGE_analysis):
		## information for MGE module analysis
		MGE.help_MGE_analysis()
		exit()

	elif (options.help_input_MGE):
		## information for PhiSpy
		MGE.help_input_MGE()
		exit()

	### 
	functions.pipeline_header()
	functions.boxymcboxface("BacterialTyper analysis")

	print ("--------- Starting Process ---------")
	functions.print_time()

	## absolute path for in & out
	input_dir = os.path.abspath(options.input)
	outdir=""

	## set mode: project/detached
	if (options.project):
		outdir = input_dir		
	elif (options.detached):
		outdir = os.path.abspath(options.output_folder)



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
This code prepares the database information for further analysis.
Several functions are implemented for:
	
	- Manipulate data entries, update information
	
	- Download NCBI assembly IDs provided
"""
## useful imports
import time
import io
import os
import re
import sys
import pandas as pd
import ncbi_genome_download as ngd
import shutil
from sys import argv
from io import open
from termcolor import colored
from Bio import SeqIO
import concurrent.futures

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import ariba_caller
from BacterialTyper.scripts import species_identification_KMA
from BacterialTyper.scripts import min_hash_caller

##########################################################################################
def NCBI_DB(strains2get, data_folder, Debug):
	"""Donwloads given taxa from NCBI if not available and updates database information.
	
	This function checks in the given folder if strain of interest is available. If not it would connect to NCBI using python module ncbi_genome_download and downloads some information.
	
	:param strains2get: dataframe containing genus, species and NCBI assembly columns among others. See example below.
	:param data_folder: Absolute path to database NCBI folder.
	:param Debug: Print messages for debugging purposes if desired. 
	:type strains2get: dataframe
	:type data_folder: string
	:type Debug: bool
	:return: Dataframe of genbank database updated for all available entries.

	Columns for the dataframe :file:`strains2get` consist of:
	
	sample,genus,species,strain,BioSample,genome,Plasmids
 
	See and example in file: :file:`/devel/results/strains2get_NCBI_DB.csv` and shown here:
	
	.. include:: ../../devel/results/strains2get_NCBI_DB.csv
		:literal:
		
	See example of the return dataframe, containing database information updated in file: :file:`/devel/results/genbank_database.csv` here:
	
	.. include:: ../../devel/results/genbank_database.csv
		:literal:
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.create_folder`
	
		- :func:`BacterialTyper.scripts.functions.get_data`
	
		- :func:`BacterialTyper.scripts.database_generator.get_dbs`
	
		- :func:`BacterialTyper.scripts.database_generator.get_database`
		
		- :func:`BacterialTyper.scripts.database_generator.NCBIdownload`
		
		- :func:`BacterialTyper.scripts.database_generator.update_db_data_file`
		
	.. include:: ../../links.inc	 	
	
	"""
	
	## set index
	strains2get = strains2get.set_index('NCBI_assembly_ID', drop=False) ## set new index but keep column
	strains2get.index.names = ['ID'] ## rename index
	strains2get = strains2get.drop_duplicates()
	
	#########
	if Debug:
		print (colored("DEBUG: NCBI data provided: ", 'yellow'))
		print (strains2get)

	## get data existing database
	print ("+ Create the database in folder: \n", data_folder)
	functions.create_folder(data_folder)
	
	## read database 
	db_frame = getdbs('NCBI', data_folder, 'genbank', Debug)
	database_df = get_database(db_frame, Debug)
	
	#########
	if Debug:
		print (colored("DEBUG: NCBI genbank database retrieved: ", 'yellow'))
		print("db_frame")
		print(db_frame)
		print()
		
		print ("database_df")
		print (database_df)
	
	## loop and download
	for index, row in strains2get.iterrows():
		functions.print_sepLine("+", 75, False)
		acc_ID = index #strains2get.loc[index]['NCBI_assembly_ID']
		info = "Genus: " + strains2get.loc[index]['genus'] + '\n' + "Species: " +  strains2get.loc[index]['species'] + '\n' + "Strain: " +  strains2get.loc[index]['name'] + '\n' + "ID accession: " +  acc_ID + '\n'
		dir_path = data_folder + '/genbank/bacteria/' + acc_ID ## module ngd requires to download data in bacteria subfolder under genbank folder

		## check if already exists
		if acc_ID in database_df.index:
			print ("\n+ Data is already available in database for: ")
			print (colored(info, 'green'))

		else:
			## download
			print ("\n+ Downloading data for:")	
			print (colored(info, 'green'))
			data_accID = NCBIdownload(acc_ID, strains2get, data_folder)
			this_db = functions.get_data(data_accID, ',', 'index_col=0')
			this_db = this_db.set_index('ID')
			database_df = database_df.append(this_db)

	## Generate/Update database
	database_csv = data_folder + '/genbank_database.csv'
	db_updated = update_db_data_file(database_df, database_csv)
	print ("+ Database has been generated in file: ", database_csv)
	return (db_updated)

##########################################################################################
def NCBI_descendant(tax_ID, NCBI_folder, Debug):
	print ()
	
	## [TODO]	
	## get information from NCBItaxonomy from module ete3
	## and generate a dataframe containing as header: genus,species,name,NCBI_assembly_ID
	## and call NCBI_DB to download and update database, then returned dataframe.


def ngd_download(dir_path, acc_ID, data_folder):
	download = False
	print ('+ Check data for ID: ', acc_ID)
	if os.path.exists(dir_path):
		print ('+ Folder already exists: ', dir_path)
		## get files download
		(genome, prot, gff, gbk) = get_files_download(dir_path)
		if all([genome, prot, gff, gbk]):
			download = False
		else:
			print ('+ Not all necessary data is available. Download it again.')
			download = True
	else:
		download = True
	
	if download:
		print ('+ Downloading:')
		## download in data folder provided
		ngd.download(section='genbank', file_formats='fasta,gff,protein-fasta,genbank', assembly_accessions=acc_ID, output=data_folder, groups='bacteria')

		## check if files are gunzip
		files = os.listdir(dir_path)
		files_list = []		
		for f in files:
			if f.endswith('gz'):
				files_list.append(f)
				print ("\t- Extracting files: ", f)
				functions.extract(dir_path + '/' + f, dir_path)
				#os.remove(dir_path + '/' + f)
	else:
		print ('+ Data is already available, no need to download it again')


##########################################################################################
def NCBIdownload(acc_ID, data, data_folder):	
	
	## module ngd requires to download data in bacteria subfolder under genbank folder
	dir_path = os.path.join(data_folder, 'genbank', 'bacteria', acc_ID) 
	ngd_download(dir_path, acc_ID, data_folder)
	
	## get files download
	(genome, prot, gff, gbk) = get_files_download(dir_path)

	## check if any plasmids downloaded
	plasmid_count = 0
	plasmid_id = []
	contig_out_file = dir_path + '/' + acc_ID + '_chromosome.fna'
	plasmid_out_file = dir_path + '/' + acc_ID + '_plasmid.fna' 
	
	## open
	contig_out_file_handle = open(contig_out_file, 'w')
	for seq_record in SeqIO.parse(genome, "fasta"):
		plasmid_search = re.search(r".*plasmid.*", seq_record.description)
		if plasmid_search:
			## count and get names for plasmids
			plasmid_count += 1
			name = str( seq_record.id )
			plasmid_id.append(name)
		
			### Separate plasmids from main sequence
			plasmid_out_file_handle = open(plasmid_out_file, 'a')
			plasmid_out_file_handle.write(seq_record.format("fasta"))
			plasmid_out_file_handle.write('\n')
			plasmid_out_file_handle.close()
		else:
			contig_out_file_handle.write(seq_record.format("fasta"))
			contig_out_file_handle.write('\n')
			contig_out_file_handle.close()

	## no plasmids found
	if plasmid_count == 0:
		plasmid_out_file = ""
		
	data2download=pd.DataFrame(columns=('ID','folder','genus','species','name','genome', 'chr', 'GFF','GBK', 'proteins','plasmids_number','plasmids_ID','plasmids'))
	data2download.loc[len(data2download)] = (acc_ID, dir_path, data.loc[acc_ID]['genus'], 
											data.loc[acc_ID]['species'], data.loc[acc_ID]['name'], genome, 
											contig_out_file, gff, prot, gbk,
											plasmid_count, "::".join(plasmid_id), plasmid_out_file)

	## dump to file
	info_file = dir_path + '/info.txt'
	data2download.to_csv(info_file)
	
	## timestamp
	filename_stamp = dir_path + '/.success'
	stamp =	functions.print_time_stamp(filename_stamp)

	## return data
	return(info_file)		
	
##########################################################################################
def get_files_download(folder):
	## check if files are gunzip
	files = os.listdir(folder)
	genome=""
	prot=""
	gff=""
	gbk=""
	for f in files:
		if f.endswith('genomic.fna'):
			genome = os.path.join(folder, f)
		elif f.endswith('genomic.gff'):
			gff = os.path.join(folder, f)
		elif f.endswith('genomic.gbk'):
			gbk = os.path.join(folder, f)
		elif f.endswith('genomic.gbff'):
			gbk = os.path.join(folder, f)
		elif f.endswith('protein.faa'):
			prot = os.path.join(folder, f)

	return(genome, prot, gff, gbk)			

##########################################################################################
def get_database(db_frame, Debug):
	data4db = pd.DataFrame()
	for index, row in db_frame.iterrows():
		## information
		this_file = db_frame.loc[index]['path'] + '/info.txt'
		if os.path.isfile(this_file):
			print ('+ Reading information for sample: ', db_frame.loc[index]['db'])
			print (colored("\t+ Obtaining information from file: %s" %this_file, 'yellow'))
			this_db = functions.get_data(this_file, ',', 'index_col=0')
			data4db = data4db.append(this_db)
			timestamp = db_frame.loc[index]['path'] + '/.success'
			if os.path.isfile(timestamp):
				stamp =	functions.read_time_stamp(timestamp)
				print (colored("\t+ Data generated on: %s" %stamp, 'yellow'))

			functions.print_sepLine("*",25, False)

	## index by ID
	if not data4db.empty:
		data4db = data4db.set_index('ID')

	return(data4db)

##########################################################################################
def update_db_data_file(data, csv):
	if os.path.isfile(csv):
		print ("\n+ Updating database")
		print ("+ Obtaining information from database file: %s" %csv)
		db2update = functions.get_data(csv, ',', 'index_col=0')
		
		## TODO: provide preference to db2update
		df = pd.concat([db2update, data], join='inner', sort=True).drop_duplicates()
		df.to_csv(csv)
		return (df)
	else:
		data.to_csv(csv)
		return (data)
		
##################################################
def getdbs(source, database_folder, option, debug):
	"""Get databases available within the folder provided.
	
	:param source: Type of database to search: ARIBA, KMA, NCBI, MLST, user_data
	:param database_folder: Absolute path to database folder.
	:param option: String containing multiple entries separated by '#' that indicate the type of database entries to search within each source type.
	:param debug: True/False for debugging messages.
	
	:type source: string
	:type database_folder: string
	:type option: string
	:type debug: bool
	
	:returns: Dataframe containing absolute paths to the available databases for each type requested. It contains columns for: "source", "db", "path"
		
	e.g.: 	source = KMA
			option = kma:archaea,plasmids,bacteria#kma_external:/path/to/file1,/path/to/file2#user_data#genbank **
			
	e.g.: 	source = NCBI
			option = genbank
	
	"""
	
	## init dataframe
	colname = ["source", "db", "path"]
	db_Dataframe  = pd.DataFrame(columns = colname)

	## read folders within database
	if os.path.isdir(database_folder):
		files = os.listdir(database_folder) ## ARIBA/KMA_db/genbank/user_data
	else:
		return db_Dataframe

	## debug message
	if (debug):
		print (colored("Folders: " + str(files),'yellow'))
		print ()
	
	## user input
	dbs2use = []
	option_list = option.split("#")
	
	for option_item in option_list:
		
		## debug message
		if (debug):
			print (colored("Option item: " + option_item,'yellow'))
		
		###
		dbs2use_tmp = []
		
		## kma
		if (option_item.startswith('kma')):
			if (option_item.startswith('kma:')):
				dbs2use_tmp = option_item.split(":")[1].split(",")

			elif (option_item.startswith('kma_external:')):
				external = option_item.split(":")[1].split(",")

				## add to dataframe			
				for ext in external:
					name_ext = os.path.basename(ext)
					db_Dataframe.loc[len(db_Dataframe)] = ['KMA_External', name_ext, ext]

			elif (option_item.startswith('kma_user_data:')):
				dbs2use_tmp = option_item.split(":")[1].split(",")
			
			elif (option_item.startswith('kma_NCBI:')):
				dbs2use_tmp = option_item.split(":")[1].split(",")
			
		### ARIBA
		elif (option_item.startswith('ARIBA:')):
			dbs2use = option_item.split(":")[1].split(",")
		
		### NCBI: genbank
		elif (option_item.startswith('genbank')):
			dbs2use.append('genbank')
		
		### NCBI: taxonomy ID
		elif (option_item.startswith('tax_id')):
			dbs2use.append('taxonomy_id')
		
		### user_data
		elif (option_item.startswith('user_data')):
			dbs2use.append('user_data')
		
		### MLST
		elif (option_item.startswith('MLST')):
			dbs2use_tmp = option_item.split(":")[1].split(",")

		### Mash
		elif (option_item.startswith('Mash')):
			if (option_item.startswith('Mash_external_data:')):
				external = option_item.split(":")[1].split(",")
				## add to dataframe			
				for ext in external:
					name_ext = os.path.basename(ext)
					name_ext_ = name_ext.split('.fna')[0]
					db_Dataframe.loc[len(db_Dataframe)] = ['Mash_external', name_ext_, ext]
			else:
				dbs2use_tmp = option_item.split(":")[1].split(",")

		### Other?
		else:
			dbs2use.append(option_item) ## add ARIBA, user_data or genbank option if provided

		## get all		
		dbs2use = dbs2use + dbs2use_tmp

	## debug message
	if (debug):
		print (colored("\ndbs2use:\n\t" + "\n\t".join(dbs2use), 'yellow'))

	## init dataframe
	#colname = ["source", "db", "path"]
	#db_Dataframe  = pd.DataFrame(columns = colname)
	
	###############
	#### ARIBA ####
	###############
	if (source == 'ARIBA'):
		### Check if folder exists
		functions.create_subfolder('ARIBA', database_folder)
		
		### get information
		ARIBA_dbs = ariba_caller.get_ARIBA_dbs(dbs2use) ## get names
		for ariba_db in ARIBA_dbs:
			this_db = database_folder + '/ARIBA/' + ariba_db + '_prepareref/'
			if os.path.exists(this_db):
				code_check_db = ariba_caller.check_db_indexed(this_db, 'NO')
				if (code_check_db == True):
					db_Dataframe.loc[len(db_Dataframe)] = ['ARIBA', ariba_db, this_db]
					print (colored("\t- ARIBA: including information from database: " + ariba_db, 'green'))
			else:
				print ("+ Database: ", ariba_db, " is not downloaded...")
				print ("+ Download now:")
				folder_db = functions.create_subfolder(ariba_db, database_folder + '/ARIBA')
				code_db = ariba_caller.ariba_getref(ariba_db, folder_db, debug, 2) ## get names 
				if (code_db == 'OK'):
					db_Dataframe.loc[len(db_Dataframe)] = ['ARIBA', ariba_db, this_db]
					print (colored("\t- ARIBA: including information from database: " + ariba_db, 'green'))

	#############
	#### KMA ####
	#############
	elif (source == 'KMA'):
		### Check if folder exists
		KMA_db_abs = functions.create_subfolder('KMA_db', database_folder)
		kma_dbs = os.listdir(KMA_db_abs)

		## debug message
		if (debug):
			print (colored("Folders KMA_db:" + str(kma_dbs) , 'yellow'))

		### get information
		for db in dbs2use:
			this_db = KMA_db_abs + '/' + db
			
			#### genbank	
			if (db == "genbank"):
				## KMA databases exists
				this_db_file = this_db + '/genbank_KMA'
				if os.path.isfile(this_db_file + '.comp.b'):
					print (colored("\t- genbank: including information from different reference strains available.", 'green')) ## include data from NCBI
					db_Dataframe.loc[len(db_Dataframe)] = ['KMA_genbank', 'genbank', this_db_file]
		
			#### user_data
			elif (db == "user_data"):
				## KMA databases exists
				this_db_file = this_db + '/userData_KMA'
				if os.path.isfile(this_db_file + '.comp.b'):
					print (colored("\t- user_data: including information from user previously generated results", 'green')) ## include user data
					db_Dataframe.loc[len(db_Dataframe)] = ['KMA_user_data', 'user_data', this_db_file]
					
			
			## default KMA databases: bacteria & plasmids
			else:
				##
				if (db == 'plasmids'):
					prefix = '.T'
				elif (db == 'viral'):
					prefix = '.TG'
				else:
					prefix = '.ATG'

				this_db_file = this_db + '/' + db + prefix
				if os.path.isfile(this_db_file + '.comp.b'):
					db_Dataframe.loc[len(db_Dataframe)] = ['KMA_db', db, this_db_file]
					print (colored("\t- KMA: including information from database " + db, 'green'))
				else:
					print (colored("\t**KMA: Database %s was not available." %db, 'red'))

					## if missing: call download module
					print ("+ Download missing KMA_db (%s) provided" %db)
					species_identification_KMA.download_kma_database(database_folder + '/KMA_db/' + db, db, debug)

					if os.path.isfile(this_db_file + '.comp.b'):
						db_Dataframe.loc[len(db_Dataframe)] = ['KMA_db', db, this_db_file]
						print (colored("\t- KMA: including information from database " + db, 'green'))
					else:
						print (colored("\t**KMA: Database %s was not available." %db, 'red'))

	##############
	#### NCBI ####
	##############
	elif (source == 'NCBI'):
		
		## TODO: get additional information from 
		## info_file = dir_path + '/info.txt'

		### Check if folder exists
		path_genbank = os.path.join(database_folder, source, 'genbank')
		db2use_abs = functions.create_subfolder(dbs2use[0], database_folder)
		
		### genbank entries downloaded
		if dbs2use[0] == 'genbank':
			##
			if os.path.exists(path_genbank + '/bacteria'):
				genbank_entries = os.listdir(os.path.join(path_genbank, 'bacteria'))
				for entry in genbank_entries:
					this_db = os.path.join(path_genbank,'bacteria', entry)
					db_Dataframe.loc[len(db_Dataframe)] = ['NCBI:genbank', entry, this_db]

		elif dbs2use[0] == 'tax_id':		
			tax_id_entries = db2use_abs

	###################
	#### user_data ####
	###################
	elif (source == 'user_data'):
		### Check if folder exists
		db2use_abs = functions.create_subfolder(dbs2use[0], database_folder)

		user_entries = os.listdir(db2use_abs)
		for entry in user_entries:
			this_db = db2use_abs + '/' + entry
			db_Dataframe.loc[len(db_Dataframe)] = ['user_data', entry, this_db]
	
	#################
	#### PubMLST ####
	#################
	elif (source == 'MLST'):
		### get information
		for db in dbs2use:
			if db == 'PubMLST':
				### Check if folder exists
				db2use_abs = functions.create_subfolder('PubMLST', database_folder)
				list_profiles = os.listdir(db2use_abs)

				for entry in list_profiles:
					this_db = db2use_abs + '/' + entry
					db_Dataframe.loc[len(db_Dataframe)] = ['MLST', 'PubMLST', entry + ',' + this_db]
					print (colored("\t- MLST: including information from profile: " + entry, 'green'))
					
			else:
					db_Dataframe.loc[len(db_Dataframe)] = ['MLST', 'user_profile', db]
					print (colored("\t- MLST: including information from profile provided by user: " + db, 'green'))

	##################
	#### Min Hash ####
	##################
	elif (source == 'MASH'):

		db_Dataframe['original'] = ''
		db_Dataframe['ksize'] = ''
		db_Dataframe['num_sketch'] = ''
		db_Dataframe['folder'] = ''
	
		### get information
		for db in dbs2use:
			#### genbank	
			if (db == "genbank"):
				
				### Check if folder exists
				db2use_abs = database_folder + '/NCBI/genbank/bacteria'
				if os.path.exists(db2use_abs):
					print (colored("\n\t- genbank: including information from different reference strains available.", 'green')) ## include data from NCBI
					genbank_entries = os.listdir(db2use_abs)
					for entry in genbank_entries:
						print ('\t+ Reading information from sample: ', entry)
						this_db = db2use_abs + '/' + entry
						
						## get additional information from 
						info_file = this_db + '/info.txt'
						info_data = pd.read_csv(info_file).set_index('ID')
						
						info_data.fillna("NaN", inplace=True)
						
						## get readable name for each strain
						entry_strain = str(info_data.loc[entry]['name'])
						
						if entry_strain == 'NaN': ## TODO: debug if it works
							entry_strain = entry
							print()
						else:
							print ('\t\t+ Rename into: ', entry_strain)
													
																	 
						list_msh = functions.retrieve_matching_files(this_db, '.sig')
						if (list_msh):
							## print original in file
							file2print = this_db + '/.original'
							if not os.path.exists(file2print):
								original = ['NaN']
							else:
								original = functions.readList_fromFile(file2print)

							db_Dataframe.loc[len(db_Dataframe)] = ['genbank', entry_strain, list_msh[0], this_db + '/mash/' + original[0], original[1], original[2], this_db]
						else:
							## index assembly or reads...
							list_fna = functions.retrieve_matching_files(this_db, 'genomic.fna')

							## not available
							db_Dataframe.loc[len(db_Dataframe)] = ['genbank', entry_strain, 'NaN', list_fna[0], 'NaN', 'NaN', this_db]

			#### user_data
			elif (db == "user_data"):
				print (colored("\n\t- user_data: including information from user previously generated results", 'green')) ## include user data
				db2use_abs = functions.create_subfolder('user_data', database_folder)
				user_entries = os.listdir(db2use_abs)
				for entry in user_entries:
					if entry == 'user_database.csv':
						continue
					
					print ('\t+ Reading information from sample: ', entry)
					this_db = db2use_abs + '/' + entry
					this_mash_db = this_db + '/mash/' + entry + '.sig'
					if os.path.exists(this_mash_db):
						## print original in file
						file2print = this_db + '/mash/.original'
						if not os.path.exists(file2print):
							original = ['NaN', 'NaN', 'NaN']
						else:
							original = functions.readList_fromFile(file2print)

						##
						db_Dataframe.loc[len(db_Dataframe)] = ['user_data', entry, this_mash_db, this_db + '/mash/' + original[0], original[1], original[2], this_db + '/mash']
					else:
						## not available
						list_fna = functions.retrieve_matching_files(this_db + '/assembly', '.fna')
						db_Dataframe.loc[len(db_Dataframe)] = ['user_data', entry, 'NaN', list_fna[0], 'NaN', 'NaN', this_db + '/mash']

	#### external_data
	### TODO: Fix this
	mash_bin = "" #set_config.get_exe('mash')
	if any(name in 'Mash_external' for name in db_Dataframe['source'].to_list()):
		print (colored("\t- external_data: including information from external data provided by user", 'green')) ## include user data
		db_Dataframe = db_Dataframe.set_index("db", drop = False)
		frame = db_Dataframe[ db_Dataframe['source'] == 'Mash_external' ]
		for index, row in frame.iterrows():
			print ('\t+ Reading information for file: ', row['db'])
			outfile = row['path'] + '.msh'
			if not os.path.exists(outfile):
				path_file = os.path.dirname(row['path'])
				this_db_file = min_hash_caller.sketch_database([row['path']], mash_bin, row['path'], row['db'], path_file)
				functions.print_sepLine("*",50, False)

			db_Dataframe.loc[row['db']] = ['Mash_external', row['db'], outfile, row['path']]
	
	## index by id	
	db_Dataframe = db_Dataframe.set_index("db", drop = False)			
	return (db_Dataframe)



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Generates a search among NCBI taxonomy and retrieves available genomes of interest
"""

## Original code extracted from: ncbi-genome-download contribution script
## https://raw.githubusercontent.com/kblin/ncbi-genome-download/master/contrib/gimme_taxa.py

## useful imports
import time
import io
import os
import re
import sys
import pandas as pd

from sys import argv
from io import open
from termcolor import colored
from ete3 import NCBITaxa

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import edirect_caller

##########################################################################################
def NCBItaxa_db(dbfile_path):
	
	# Generate the ete3 NCBI taxa object
	ncbi = NCBITaxa(dbfile_path)

	## check days passed
	#ncbi.update_taxonomy_database()
	
	
##########################################################################################
def get_descendant_taxid(taxid_name, dbfile_path):
	## use NCBI taxa class from ete3

	# Get NCBI taxa object
	ncbi = NCBITaxa_db(dbfile_path)
	
	## Look for taxid name provided
	taxid = ncbi.get_name_translator([taxid_name])[taxid_name][0]

	# Get all taxa from a given group
	descendent_taxa = ncbi.get_descendant_taxa(taxid)
	descendent_taxa_names = ncbi.translate_to_names(descendent_taxa)





#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Calls Trimmomatic for the trimming of sequence adapter within fastq reads.
"""

## useful imports
import time
import io
import os
import re
import subprocess
import sys
from sys import argv
from termcolor import colored

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

################################################
def trimmo_module(files, path_name, sample_name, threads, Debug, trimmomatic_adapters):
	## 
	## This functions generates a trimmomatic call using java and trimmomatic from 
	## the system with a minimum version (specified in config.py)
	## Checks if adapter file exists
	## Returns code from trimmo_call: OK/FAIL
	##
	
	## get exe
	trimmomatic_jar = set_config.get_exe('trimmomatic')
	java_path = set_config.get_exe('java')

	## check if it exists
	if os.path.isfile(trimmomatic_adapters):
		## debug message
		if (Debug):
			print (colored("**DEBUG: trimmomatic_adapters file exists **", 'yellow'))
			print (trimmomatic_adapters)
	else:
		## rise error & exit
		print (colored("***ERROR: Trimmomatic adapters file does not exist: " + trimmomatic_adapters,'red'))
		exit()
	
	## call
	return(trimmo_call(java_path, path_name, sample_name, files, trimmomatic_jar, threads, trimmomatic_adapters, Debug))

################################################
def print_help_adapters():
	"""Trimmomatic adapters information
	
	BacterialTyper includes a file :file:`BacterialTyper.data.available_Trimmomatic_adapters.fasta` with default sequencing adapters provided by Trimmomatic_ (v0.39).
	
	User can provide using the option --adapters different sequencing adapters.
	
	.. seealso:: Additional information on Trimmomatic adapters available.
	
		- :doc:`Trimmomatic adapters <../../../data/trimmomatic_adapters>` 
	
	.. include:: ../../links.inc 
	"""
	## [TODO]
	print (colored("\n\n ***** TODO: Generate this help message *****\n\n", 'red'))
	
	print("BacterialTyper includes a file with default sequencing adapters provided by Trimmomatic (v0.39).")
	print("See additional details in the documentation.\n")
	
	print("Users can provide any sequencing adapter of interest using the option --adapters in fasta format file.")

################################################
def trimmo_call(java_path, sample_folder, sample_name, files, trimmomatic_jar, threads, trimmomatic_adapters, Debug):
	##
	## Function to call trimmomatic using java. Can take single-end and pair-end files
	## sample_folder must exists before calling this function. 
	## It can be call from main or a module.
	## Returns code OK/FAIL according if succeeded or failed the system call
	## 

	#######################################
	## http://www.usadellab.org/cms/?page=trimmomatic
	#
	# ILLUMINACLIP:fasta_file.fa:2:30:10 LEADING:11 TRAILING:11 SLIDINGWINDOW:4:20 MINLEN:24
	#
	# This will perform the following:
	#	Remove adapters (ILLUMINACLIP:fasta_file.fa:2:30:10)
	#	Remove leading low quality or N bases (below quality 11) (LEADING:11)
	#	Remove trailing low quality or N bases (below quality 11) (TRAILING:11)
	#	Scan the read with a 4-base wide sliding window, cutting when the average quality per base drops below 20 (SLIDINGWINDOW:4:20)
	#	Drop reads below the 24 bases long (MINLEN:24)
	#######################################


	## debug message
	if (Debug):
		print (colored("+ Cutting adapters for sample: " + sample_name, 'yellow'))
		
	## log files
	log_file = sample_folder + '/' + sample_name + '_call.log'
	trimmo_log = sample_folder + '/' + sample_name + '.log'
	
	## init
	file_R1 = ""
	file_R2 = ""
	trim_R1 = ""
	orphan_R1 = ""
	trim_R2 = ""
	orphan_R2 = ""

	## Paired or single end
	## set command
	cmd = ""	
	if (len(files) == 2): ## paired-end
		file_R1 = files[0]
		file_R2 = files[1]

		#print ('\t-', file_R2)
		trim_R1 = sample_folder + '/' + sample_name + '_trim_R1.fastq'
		orphan_R1 = sample_folder + '/' + sample_name + '_orphan_R1.fastq'
		trim_R2 = sample_folder + '/' + sample_name + '_trim_R2.fastq'
		orphan_R2 = sample_folder + '/' + sample_name + '_orphan_R2.fastq'

		cmd = "%s -jar %s PE -threads %s -trimlog %s %s %s %s %s %s %s ILLUMINACLIP:%s:2:30:10 LEADING:11 TRAILING:11 SLIDINGWINDOW:4:20 MINLEN:24 2> %s" %(java_path, trimmomatic_jar, threads, log_file, file_R1, file_R2, trim_R1, orphan_R1, trim_R2, orphan_R2, trimmomatic_adapters, trimmo_log)

	else: ## single end
		file_R1 = files[0]
		trim_R1 = sample_folder + '/' + sample_name + '_trim.fastq'

		cmd = "%s -jar %s SE -threads %s -trimlog %s %s %s ILLUMINACLIP:%s:2:30:10 LEADING:11 TRAILING:11 SLIDINGWINDOW:4:20 MINLEN:24 2> %s" %(java_path, trimmomatic_jar, threads, log_file, file_R1, trim_R1, trimmomatic_adapters, trimmo_log)

	## system call & return
	code = functions.system_call(cmd)
	if code == 'OK':
		## success stamps
		filename_stamp = sample_folder + '/.success'
		stamp =	functions.print_time_stamp(filename_stamp)	
		return('OK')	
	else:
		return('FAIL')	


################################################
def	help_options():
	print ("\nUSAGE:\npython %s Outfolder file_R1 file_R2 trimmomatic threads trimmomatic_adapters sample_name\n"  %os.path.abspath(argv[0]))
	print ("\n*** If not paired-end provide 'na' for file_R2")

################################################
def main():

  	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()    	

	## ARGV
	path_name = os.path.abspath(argv[1])
	file_R1 = os.path.abspath(argv[2])
	file_R2 = os.path.abspath(argv[3])
	trimmomatic_jar = argv[4]
	threads = argv[5]
	trimmomatic_adapters = os.path.abspath(argv[6]) ##"/imppc/labs/lslab/share/data/references/Trimmomatic_adapters.fa"
	sample_name = argv[7]

	## call
	trimmo_call(path_name, sample_name, file_R1, file_R2, trimmomatic_jar, threads, trimmomatic_adapters)
		
################################################

'''******************************************'''
if __name__== "__main__":
	main()
		




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
'''
Calls sourmash_ and generates signatures for each genome, clusterizes and generates plots. 

.. include:: ../../links.inc	 
'''
#################################################
## code taken and adapted from: 
##	https://sourmash.readthedocs.io/en/latest/api-example.html
## 	https://github.com/dib-lab/sourmash/blob/master/sourmash/commands.py
#################################################

## useful imports
import time
import io
import os
import re
import sys
import pandas as pd
from sys import argv
from io import open
from termcolor import colored
from Bio import Phylo
from io import StringIO

## import modules sourmash
import csv
import shutil
import sourmash
import screed
import numpy
from sourmash import SourmashSignature, save_signatures, load_one_signature
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

import pylab
import scipy.cluster.hierarchy as sch

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import database_generator

##################################################
def helpMash():
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))

##################################################		
def sketch_database(dict_files, folder, Debug, ksize_n, num_sketch):	
	"""Sketch sequence files
	
	This function generates a sourmash index, also called sketch, of the sequences 
	provided in the folder specified.
	
	For speed reasons, we set force=True in add_sequence step to skip over k-mers containing 
	characters other than ACTG, rather than raising an exception.

	:param dict_files: keys are the names of the files and values are the path to the fasta file
	:param folder:
	:param Debug: True/False to print developing messages.
	:param ksize_n: Kmer size value.
	:param num_sketch: Number of sketches to include in the hash signature. 
	
	:type dict_files: Dictionary
	:type folder: string 
	:type Debug: bool
	:type ksize_n: integer
	:type num_sketch: integet
	
	:returns: List of SourmashSignature signatures (siglist) and absolute path files generated (siglist_file).  
	
	
	.. attention:: The code to implement this API function was taken and adapted from: 
	 
		- https://sourmash.readthedocs.io/en/latest/api-example.html
	
		- https://github.com/dib-lab/sourmash/blob/master/sourmash/commands.py
		
	
	.. seealso:: This function depends on sourmash python module (https://sourmash.readthedocs.io/en/latest/). Some functions employed are:
	
		- :func:`sourmash.MinHash`
		
		- :func:`sourmash.SourmashSignature`
		
		- :func:`sourmash.MinHash.add_sequence`
		
		
	.. include:: ../../links.inc	 
	
	"""
	### Default: set as option
	## num_sketch=5000
	## ksize_n=31
	
	minhashes = {}
	for name,g in dict_files.items():
		print ('\t+ Skecthing sample: ', name)
		E = sourmash.MinHash(n=num_sketch, ksize=ksize_n)	## generate hash according to number of sketches and kmer size
		for record in screed.open(g):
			E.add_sequence(record.sequence, True)
		## in add_sequence and for speed reasons, we set force=True to skip over k-mers containing characters other than ACTG, rather than raising an exception.
		minhashes[name]= E
		
	## Debug messages
	if Debug:
		print (colored("\n*** DEBUG: minhashes *****\n", 'red'))
		print (type(minhashes))	
		print (minhashes)

	siglist = []
	siglist_file = []

	### save as signature
	functions.create_folder(folder)
	for names,hashes in minhashes.items():
		sig1 = SourmashSignature(hashes, name=names)
		outfile_name = folder + '/' + str(names) + '.sig'
		with open(outfile_name, 'wt') as fp:
			save_signatures([sig1], fp)

		siglist_file.append(outfile_name)
		siglist.append(sig1)
	
	return(siglist_file, siglist)		
	
##################################################		
def read_signature(sigfile, ksize_n):
	"""Loads signatures stored in files
	
	Signature generated are stored as MinHashes in JSON format file. Sourmash implements a function to load signatures.
	
	This function here is just a shortcut for the :func:`sourmash.load_one_signature` function in sourmash_.
	
	:param sigfile: Hash signature stored in JSON format
	:param ksize_n: Kmer size.
	
	:type sigfile: string
	:type ksize_n: integer
	
	
	.. seealso:: This function depends on sourmash python module (https://sourmash.readthedocs.io/en/latest/). Some functions employed are:
	
		- :func:`sourmash.load_one_signature`
		
	
	.. include:: ../../links.inc	 
	"""
	
	## code taken and adapted from: https://sourmash.readthedocs.io/en/latest/api-example.html
	#print ('\t+ Loading signature for comparison...')
	sig = load_one_signature(sigfile, ksize=ksize_n, select_moltype='DNA', ignore_md5sum=False)
	return (sig)

##################################################
def compare(siglist, output, Debug):
	#################################################
	## code taken and adapted from: 
	##	https://sourmash.readthedocs.io/en/latest/api-example.html
	## 	https://github.com/dib-lab/sourmash/blob/master/sourmash/commands.py
	#################################################
   
	# build the distance matrix
	D = numpy.zeros([len(siglist), len(siglist)])
	numpy.set_printoptions(precision=3, suppress=True)

	# do all-by-all calculation
	labeltext = []
	for i, E in enumerate(siglist):
		for j, E2 in enumerate(siglist):
			if i < j:
				continue
			similarity = E.similarity(E2, False) ## False -> ignore abundances
			D[i][j] = similarity
			D[j][i] = similarity

		labeltext.append(E.name())

   	## Debug messages
	if Debug:
		print (colored("\n*** DEBUG: compare minhashes *****\n", 'red'))
		print (type(D))
		print ('D:')
		print (D)
		print ("Labeltext:")
		print (labeltext)
		print ('Min similarity in matrix: {:.3f}', numpy.min(D)) ## use this to color accordingly
		
	### Write output
	labeltext_string = [str(x) for x in labeltext] ## generat strings, avoid if integers alone
	labeloutname = output + '.labels.txt'
	with open(labeloutname, 'w') as fp:
		fp.write("\n".join(labeltext_string))
		
	## save matrix in csv format file
	numpy.savetxt(output + ".matrix.csv", D, delimiter=",", header= ",".join(labeltext_string) )

	## Debug messages
	if Debug:
		print('saving labels:', labeltext_string)
		print('saving labels to:', labeloutname)
		print('saving distance matrix (binary file) to:', output)
		print('saving distance matrix (csv file) to:', output)

	## return numpy array
	return (D, labeltext_string)

##################################################
def generateNewick(node, newick, parentdist, leaf_names):
	"""Generates Newick tree
	
	This functions generates a newick tree given the a tree object (output from the scipy cluster hierarchy to_tree), the labels and the corresponding matrix distance.
	
	.. note:: scipy.cluster.hierarchy.to_tree
	
		Converts a linkage matrix into an easy-to-use tree object. 
	
	:param node: Tree object (scipy.cluster.hierarchy.to_tree)
	:param newick: Newick tree format
	:param parendist: node.dist or distance matrix
	:param leaf_names: list of labels
	 
	:type newick: string
	:type parendist: matrix
	:type leaf_names: list
	 
	As generateNewick tree is called recursively to create the tree with the parents and children nodes, the first call the argument newick can be empty ("").
	
	.. attention:: The original code was taken and adapted from: 
	
		- https://github.com/scipy/scipy/issues/8274
	
		- https://stackoverflow.com/questions/28222179/save-dendrogram-to-newick-format/31878514#31878514
	
	
	.. include:: ../../links.inc	 
	"""
	
	if node.is_leaf():
		return "%s:%.2f%s" % (leaf_names[node.id], parentdist - node.dist, newick)
	else:
		if len(newick) > 0:
			newick = "):%.2f%s" % (parentdist - node.dist, newick)
		else:
			newick = ");"
		newick = generateNewick(node.get_left(), newick, node.dist, leaf_names)
		newick = generateNewick(node.get_right(), ",%s" % (newick), node.dist, leaf_names)
		newick = "(%s" % (newick)
		return newick

##################################################
def plot(D, labeltext, filename, pdf, colorLabel):
	#################################################
	## code taken and adapted from: 
	##	https://sourmash.readthedocs.io/en/latest/api-example.html
	## 	https://github.com/dib-lab/sourmash/blob/master/sourmash/commands.py
	#################################################

	matrix_out = filename + '.matrix'

	### decide on PDF/PNG output
	if pdf:
		matrix_out += '.pdf'
	
	else:
		matrix_out += '.png'
	
	###########################
	### make the histogram
	###########################
	#print ('+ Saving histogram of matrix values: ', hist_out)
	#fig = pylab.figure(figsize=(8,5))
	#pylab.hist(numpy.array(D.flat), bins=100)
	#fig.savefig(hist_out)

	#######################################
	### make the dendrogram: do clustering
	## https://docs.scipy.org/doc/scipy/reference/cluster.hierarchy.html
	#######################################
	#fig2 = pylab.figure(figsize=(11,8))
	#ax1 = fig2.add_axes([0.1, 0.1, 0.7, 0.8])
	#ax1.set_xticks([])
	#ax1.set_yticks([])
	#Y = sch.linkage(D, method='single')
	#Z1 = sch.dendrogram(Y, orientation='right', labels=labeltext)
	#fig2.savefig(dendrogram_out)
	#print ('+ Wrote dendrogram to:', dendrogram_out)

	#######################################
	### make the dendrogram+matrix:
		#######################################

	### Original code
	## fig = sourmash_fig.plot_composite_matrix(D, labeltext, show_labels=args.labels,
	##			 show_indices=args.indices, vmin=args.vmin, vmax=args.vmax, force=args.force)
	## I get code from the source code for this function and use it here.
	## Get to generate a slightly different image representation

	Y = sch.linkage(D, method='single') ## https://docs.scipy.org/doc/scipy/reference/generated/scipy.cluster.hierarchy.linkage.html
	fig3 = pylab.figure(figsize=(15, 10))
	ax1 = fig3.add_axes([0.09, 0.1, 0.2, 0.6])

	# plot dendrogram
	## https://python-graph-gallery.com/400-basic-dendrogram/
	Z1_2 = sch.dendrogram(Y, orientation='left', labels=labeltext)

	## set label colors if desired
	if colorLabel:
		sch.dendrogram(Y, orientation='left', labels=labeltext)
		ax = plt.gca()
		xlbls = ax.get_ymajorticklabels()
		for lbl in xlbls:
			lbl.set_color(colorLabel[ lbl.get_text() ])

	ax1.set_xticks([])
	xstart = 0.45
	width = 0.45
	scale_xstart = xstart + width + 0.01

	# plot matrix
	axmatrix = fig3.add_axes([xstart, 0.1, width, 0.6])

	# (this reorders D by the clustering in Z1_2)
	idx1 = Z1_2['leaves']
	D = D[idx1, :]
	D = D[:, idx1]

 	## use minimun distance matrix D to color accordingly
	min_D = numpy.min(D) ## 
	cmap_palette = ""
	if min_D > 0.8:
		cmap_palette = pylab.cm.YlGnBu
	else:
		cmap_palette = pylab.cm.YlGnBu

	# show matrix
	im = axmatrix.matshow(D, aspect='auto', origin='lower', cmap= cmap_palette, vmin=1, vmax=0)
	axmatrix.set_xticks([])
	axmatrix.set_yticks([])
	
	## https://matplotlib.org/3.1.0/api/_as_gen/matplotlib.pyplot.matshow.html
	## https://matplotlib.org/3.1.0/api/_as_gen/matplotlib.axes.Axes.imshow.html#matplotlib.axes.Axes.imshow
	## cmap=pylab.cm.RdBu
	## https://scipy-cookbook.readthedocs.io/items/Matplotlib_Show_colormaps.html

	# Plot colorbar.
	axcolor = fig3.add_axes([scale_xstart, 0.1, 0.02, 0.6])
	pylab.colorbar(im, cax=axcolor)	
	fig3.savefig(matrix_out)
	print ('+ Wrote matrix to:', matrix_out)
	return(Y)
	
##################################################

def get_Newick_tree(cluster_hierachy, DataMatrix, labeltext, output):
	"""
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.printList2file`
		
	"""
	
	tree = sch.to_tree(cluster_hierachy, False)
	Newick_tree = generateNewick(tree, "", DataMatrix, labeltext)
	
	Newick_tree_file = output + '.nwk'
	functions.printList2file(Newick_tree_file, [Newick_tree])
	
	handle = StringIO(Newick_tree)
	treePhylo = Phylo.read(handle, "newick")
	
	leaves_tree = []
	for leaf in treePhylo.get_terminals(): 
		leaves_tree.append(leaf.name)
		#print(leaf.name)
	
	## BUG: the list is printed alphabetically ordered
	Newick_tree_leaves =  output + '.leaves.txt'
	functions.printList2file(Newick_tree_leaves, leaves_tree)

def	help_options():
	print ("\nUSAGE: python %s fasta_1,ID1 fasta_2,ID2 ... fasta_n,IDn \n"  %os.path.realpath(__file__))

##################################################
def main():
	## this code runs when call as a single script

	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()
	
	file_names_dict = {}
	for i in argv:
		if i == argv[0]:
			continue
		
		file_names_dict[ i.split(',')[1] ] = i.split(',')[0]
	
	## to do: implement main function
	folder = "."
	Debug = False
	output = "test"
	pdf = True
	
	## Defaults
	ksize_n = 51
	num_sketch = 5000
	
	### create signatures and compare them
	(siglist_file, siglist) = sketch_database(file_names_dict, folder, Debug, ksize_n, num_sketch)
	(D, labeltext) = compare(siglist, output, Debug)
	
	## create plot
	Y = plot(D, labeltext, output, pdf, "")
	
	## create newick tree file
	get_Newick_tree(Y, D, labeltext, output)
	

##################################################
if __name__== "__main__":
	main()

	
##################################################		
##def sketch_database(list_files, mash_bin, out_file, name, folder):	
##	### -p threads
##	mash_cmd = ""
##	if len(list_files) == 1:
##		print ('\t+ Skecthing sample: ', name)
##		mash_cmd = '%s sketch -o %s %s' %(mash_bin, out_file, list_files[0])
##
##	else:
##		print ('\t+ Skecthing list of samples provided: ', name)
##		## batch
##		out_batch = folder + "/.batch_entries.txt"
##		functions.printList2file(out_batch, list_files)
##		mash_cmd = '%s sketch -l -o %s %s' %(mash_bin, out_file, out_batch)
##	
##	return(functions.system_call(mash_cmd))

# subsample?
#if args.subsample:
#	numpy.random.seed(args.subsample_seed)

#	sample_idx = list(range(len(labeltext)))
#	numpy.random.shuffle(sample_idx)
#	sample_idx = sample_idx[:args.subsample]

#	np_idx = numpy.array(sample_idx)
#	D = D[numpy.ix_(np_idx, np_idx)]
#	labeltext = [ labeltext[idx] for idx in sample_idx ]



'''
Main modules BacterialTyper
'''
__all__ = [
	'annot',
	'assemble',
	'citation',
	'cluster',
	'config',
	'database',
	'ident',
	'help_info',
	'MGE',
	'phylo',
	'profile',
	'prep',
	'qc',
	'run',
	'report_generation',
	'test',
	'trimm',
	'version'
]

from BacterialTyper.modules import *



#!/usr/bin/env python3
"""
This code generates a bacteriophage identification profile for each sample.
"""
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored
from Bio import SeqIO
from functools import reduce
from termcolor import colored
import argparse
import pandas
import shutil

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

## import phispy modules
import PhiSpyModules
import pkg_resources

######
def results_PhiSpy(folder, name):
	"""Parse results generated by PhiSpy
	
	This functions renames and updates files with information for a better interpretation of results. 
	
	:param folder: Absolute path to output directory containing PhiSpy results.
	:param name: Sample name id
	
	:type folder: string
	:type name: string
	
	.. note:: See PhiSpy results entry available in :ref:`PhiSpy-results`

	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.get_fullpath_list`

		- :func:`BacterialTyper.scripts.functions.create_subfolder`

		- :func:`BacterialTyper.scripts.functions.read_time_stamp`
		
		- :func:`BacterialTyper.scripts.functions.print_time_stamp`
				
	"""
	
	
	## filename stamp of the process
	filename_stamp = folder + '/.PhiSpy_results'

	# check if previously done
	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		print (colored("\tA previous command generated results on: %s [%s]" %(stamp, name), 'yellow'))
	else:	
	
		## get files
		PhiSpy_files = functions.get_fullpath_list(folder)
		
		## create sup_folder
		sup_folder = functions.create_subfolder("sup_files", folder)
		
		## remove any previous files
		sup_folder_files = functions.get_fullpath_list(sup_folder)
		if (sup_folder_files):
			for f in sup_folder_files:
				## remove original
				os.remove(f)
			
		print ("+ Parse results and sort files generated by PhiSpy")
		
		## parse files and rename
		for f in PhiSpy_files:
			baseName = os.path.basename(f)
			
			#print("#################################")
			#print(f)
			#print(baseName)
			
			if baseName == "classify.tsv":
				# move to sup_folder
				shutil.move(f, sup_folder)
				
			elif baseName == "prophage_tbl.tsv":
				# rename & add to excel
				new_name = folder + "/" + name + '_PhiSpy-classification_genes.tsv'
				## read pandas
				prophage_tsv = pandas.read_csv(f, sep="\t", header=0)
				
				# rename
				shutil.move(f, new_name)			
				
			### gff
			elif baseName == "prophage.gff3":
				# rename
				new_name = folder + "/" + name + '_PhiSpy-prophage.gff3'
				shutil.move(f, new_name)
				
			elif baseName == "prophage_coordinates.tsv":
				# rename, add header & add to excel
				columns = ("#prophage_ID", "Contig","Start","End","attL_Start","attL_End","attR_Start","attR_End","attL_Seq","attR_Seq","Longest_Repeat_flanking_phage")
				coordinates_pd = pandas.read_csv(f, sep="\t", header=None, names=columns)
	
				## print in file			
				new_name = folder + "/" + name + '_PhiSpy-prophage-coordinates.tsv'
				coordinates_pd.to_csv(new_name, sep="\t")
	
				## remove original
				os.remove(f)
				
			elif baseName == "prophage.tbl":
				## redundant information. move to sup_folder
				shutil.move(f, sup_folder)
				
	
			elif baseName == "testSet.txt":
				# move to sup_folder
				shutil.move(f, sup_folder)
				
			elif baseName == ".PhiSpy":
				continue
				
			
		## create excel
		## coordinates_pd, prophage_tsv
		name_excel = folder + '/' + name + '_bacteriophage_summary.xlsx'
		writer = pandas.ExcelWriter(name_excel, engine='xlsxwriter') 		
		prophage_tsv.to_excel(writer, sheet_name="Gene classification")
		coordinates_pd.to_excel(writer, sheet_name="Coordinates")
		
		## close
		writer.save()
		
		## print stamp time file
		stamp =	functions.print_time_stamp(filename_stamp)
		
	return ()

######
def ident_bacteriophage(gbk_file, name, output_dir, training_set, Debug=False, 
					min_contig_size=5000, window_size=30, nonprophage_genegaps=10, 
					number_phage_genes=5,randomforest_trees=500, expand_slope=False, 
					kmers_type="all", keep=False):

	"""Identify putative bacteriophages inserted.
	
	Function to call PhiSpy_ functions to generate the identification and annotation 
	of bacteriophages inserted in bacterial genomes.
	
	:param gbk_file: GenBank format file
	:param name: Sample name id
	:param output_dir: Absolute path to output directory
	:param training_set: trainingSet number id. [Default: 0 (Generic)]
	:param Debug: True/False for debugging messages [Default: False]
	:param min_contig_size: Minimum contig size (in bp) to be included in the analysis. Smaller contigs will be dropped. [Default: 5000 ]
	:param window_size: Window size of consecutive genes to look through to find phages. [Default: 30]
	:param nonprophage_genegaps: The number of non phage genes betweeen prophages. [Default: 10]
	:param number_phage_genes: Number of consecutive genes in a region of window size that must be prophage genes to be called. [Default: 5]
	:param randomforest_trees: Number of trees generated by Random Forest classifier. [Default: 500]
	:param expland_slope: Use the product of the slope of the Shannon scores in making test sets [Default: False]
	:param kmers_type: Type of kmers used for calculating Shannon scores. [all, codon, simple] [Default: all]
	:param keep: Do not delete temporary files [Default: False]

	:type gbk_file: string
	:type name: string
	:type output_dir: string
	:type training_set: integer
	:type Debug: Boolean 
	:type min_contig_size: integer
	:type window_size: integer
	:type nonprophage_genegaps: integer
	:type number_phage_genes: integer 
	:type randomforest_trees: integer
	:type expland_slope: Boolean
	:type kmers_type: string
	:type keep: Boolean

	The parameter `training_set` is the id for the training set of interest among the different available provided by PhiSpy within the file 
	:file:`data/trainingGenome_list.txt` and summarized here by :func:`BacterialTyper.scripts.bacteriophage.get_list_PhiSpy_trainingSets`.
	
	.. note:: See available training sets in :ref:`PhiSpy-training-sets`

	.. attention:: Be aware of Copyright
	
		The code implemented here is very similary to the main function under PhiSpy.py 
		main script: https://github.com/linsalrob/PhiSpy/blob/master/PhiSpy.py
		
		We basically implemented it here again to modulate and control accordingly to 
		our project and needs. Give credit to the authors accordingly.
		
		Copyrigth: 2008-2018 Sajia Akhter, Katelyn McNair, Rob Edwards, 
		
		San Diego State University, San Diego, CA

		
	.. note:: This functions relies on several PhiSpy functions such as:
	
		- :func:`PhiSpy.PhiSpyModules.SeqioFilter`
		
		- :func:`PhiSpy.PhiSpyModules.make_test_set`
		
		- :func:`PhiSpy.PhiSpyModules.call_randomforest`
		
		- :func:`PhiSpy.PhiSpyModules.make_initial_tbl`
		
		- :func:`PhiSpy.PhiSpyModules.consider_unknown`
		
		- :func:`PhiSpy.PhiSpyModules.fixing_start_end`		
			
	"""

	## filename stamp of the process
	filename_stamp = output_dir + '/.PhiSpy'

	# check if previously done
	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		print (colored("\tA previous command generated results on: %s [%s -- PhiSpy]" %(stamp, name), 'yellow'))
	else:	
		## debug message
		if (Debug):
			print (colored("**DEBUG: Call PhiSpy for sample %s " %gbk_file + "**", 'yellow'))

			print('+ Generate PhiSpy call for file %s...' %gbk_file)

		### create folder
		functions.create_folder(output_dir)
		
		## Filter to remove short contigs
		gbk_filtered = PhiSpyModules.SeqioFilter(filter(lambda x: len(x.seq) > min_contig_size, SeqIO.parse(gbk_file, "genbank")))
		ncontigs = reduce(lambda sum, element: sum + 1, gbk_filtered, 0)
		if ncontigs == 0:
			## debug message
			if (Debug):
				print (colored("**DEBUG: 100% contigs in %i are less than %i bp" 
							%(gbk_file, min_contig_size)), 'yellow')
			
			return(0)

		## get available trainingSets within PhiSpy
		sets_Available = get_list_PhiSpy_trainingSets()

		set_name = sets_Available.loc[int(training_set), "Set_Name"]
		training_set_formated = "data/" + set_name + ".txt"

		if (Debug):
			print ("\t+ Training Set selected: ", training_set)
			print ("\t+ Training Set name: ", set_name)
			print ("\t+ Training Set Genus: ", sets_Available.loc[int(training_set), "genus"])
			print ("\t+ Training Set Species: ", sets_Available.loc[int(training_set), "species"])
			print ("\t+ Training Set Tag: ", sets_Available.loc[int(training_set), "Tag"])
		
		## create argparse with arguments provided to call PhiSpy
		arg_parser = argparse.Namespace(infile=gbk_file, output_dir=output_dir,
								training_set = training_set_formated, min_contig_size = min_contig_size,
								window_size= window_size, nonprophage_genegaps=nonprophage_genegaps,
								number=number_phage_genes, randomforest_trees=randomforest_trees, quiet=True,
								expand_slope=expand_slope, kmers_type=kmers_type, keep=keep, record=gbk_filtered)
		## debug message
		if (Debug):
			print (colored("**DEBUG: Arguments for PhiSpy", 'yellow'))
			print (vars(arg_parser))	
		
		# make testing set: calculate Shanon slope
		my_make_test_flag = PhiSpyModules.make_test_set(**vars(arg_parser))
	
		## Classification
		PhiSpyModules.call_randomforest(**vars(arg_parser))
		PhiSpyModules.make_initial_tbl(**vars(arg_parser))
		
		## Consider unknown functions
		if (training_set == 'data/trainSet_genericAll.txt'): ## TODO: Debug test with genericAll (Default: 0)
			PhiSpyModules.consider_unknown(arg_parser.output_dir)

		## PhiSpy Evaluation
		PhiSpyModules.fixing_start_end(**vars(arg_parser))
		
		## when finished print time stamp in  output_dir + '/.PhiSpy'
		stamp =	functions.print_time_stamp(filename_stamp)

	## final results
	return (output_dir) ## contains results
######

def print_list_modified():
	"""Print PhiSpy training set information
	
	This function prints on screen the data information retrieved from PhiSpy package using 
	:func:`BacterialTyper.scripts.bacteriophage.get_list_PhiSpy_trainingSets`, 
			
	.. note:: See available training sets in :ref:`PhiSpy-training-sets`

	"""
	
	## get original
	data = get_list_PhiSpy_trainingSets()
	pandas.set_option('display.max_rows', data.shape[0]+1)
	print("\n\n")
	print(data)
	print("\n\n")


######
def get_list_PhiSpy_trainingSets():
	"""Get training Sets for PhiSpy calculations
	
	This function prints a modification of the information for better clarification to users of the different training 
	sets available within PhiSpy python package.
	
	It uses the pkg_resources resource_filename to retrieve the PhiSpy file :file:`data/trainingGenome_list.txt` 
	(within PhiSpy package) that contains all the available training sets pre-calculated for different 
	bacterial species/strains. It then manipulates the dataframe for better understanding using pandas.
	
	:returns: Pandas dataframe containing the file information
	
	.. note:: See available training sets in :ref:`PhiSpy-training-sets`

	"""
	## read into pandas dataframe the trainingGenome list dataset
	data = pandas.read_csv(pkg_resources.resource_filename('PhiSpyModules', "data/trainingGenome_list.txt"), 
						sep='\t', header=None, names=["trainingSet_ID", "Set_name", "Species", "Number_genomes"]).set_index("trainingSet_ID")
	
	## reformat table:
	## dataframe string split:
	data[["set", "ext1"]] = data["Set_name"].str.split(pat=".txt", n=1, expand=True)

	## Information regarding species specific training sets
	## Genomes == 1
	data1 = pandas.DataFrame()
	species_info = data.loc[data["Number_genomes"]==1]["Species"].str.split(pat="_", n=2, expand=True)
	data1["Set_Name"] = data.loc[data["Number_genomes"]==1]["set"]
	data1["Tag"] = "Single_species"
	#data1["Number_genomes"] = 1
	data1["genus"] = species_info[0]
	data1["species"] = species_info[1]
	data1["other1"] = species_info[2]
	data1[["other", "ext2"]] = data1["other1"].str.split(pat=".gb|.gbk", n=1, expand=True)
	data1.drop(columns=["other1", "ext2"], inplace = True)
	
	## Information regarding non-species_specific training sets
	## Genomes > 1
	data_more1 = data.loc[data["Number_genomes"]>1]
	
	## Generic Test set
	data0 = pandas.DataFrame()

	## ToDo set number as maximun if PhiSpy updates trainingSets and number 49 changes
	data0["Set_Name"] = data_more1.loc[data["Number_genomes"]==49]["set"] 
	data0["Tag"] = "Generic"

	#data0["Number_genomes"] = 49
	data0["genus"] = "All available"
	data0["species"] = "All available"
	data0["other"] = "All available"
	data_more1 = data_more1.drop(index=0)
	
	### More >1 & <49	
	data2_3 = pandas.DataFrame()
		
	for index, row in data_more1.iterrows():
		data2_3.at[index, "Set_Name"] = row["set"]
		data2_3.at[index, "Tag"] = "Multi_species"
		#data2_3.at[index, "Number_genomes"] = row["Number_genomes"]

		## extract elemnts of the list
		## split species info 
		species_info = [i.split('_', 2) for i in row["Species"].split(';')]
		data2_3.at[index, "genus"] = "/".join( list(set(functions.parse_sublist(species_info, 0))) )
		data2_3.at[index, "species"] = "/".join( list(set(functions.parse_sublist(species_info, 1))) )
		
		## remove genbank extension
		other_info = list( map( lambda x: x.replace('.gbk','').replace('.gb',''), list(set(functions.parse_sublist(species_info, 2))) ))
		data2_3.at[index, "other"] = "/".join(other_info)

	data_final = pandas.concat([data0, data1, data2_3], join='outer', sort=True)
	data_final["trainingSet"] = data_final.index
	order_list = ["trainingSet", "Tag", "Set_Name", "genus", "species", "other"]
	data_final = data_final[order_list]
	
	return (data_final)
	
######

######
def help_PhiSpy():
	"""Print additional information for PhiSpy
	
	PhiSpy_ is a program for identifying prophages from among microbial genome sequences. 
	
	Copyright: 2008-2018 
	
	Sajia Akhter, Katelyn McNair, Rob Edwards, San Diego State University, San Diego, CA
	
	This function also prints the list of training sets available and provided within PhySpy.
	
	.. note:: This function depends on the PhiSpy function:
	
		- :func:`PhiSpy.PhiSpyModules.print_list`
	
	.. include:: ../../links.inc	 	
	"""
	print ("\n** PhiSpy additional information **")
	print ("PhiSpy is a program for identifying prophages from among microbial genome sequences\n")
	print ("(c) 2008-2018 Sajia Akhter, Katelyn McNair, Rob Edwards, San Diego State University, San Diego, CA")
	print ("https://github.com/linsalrob/PhiSpy\n")
	
	print ("Choose among different training sets for a better phage identification:\n")
	print_list_modified()	
	
	print()
	print ("As stated by PhiSpy developers:")
	print ("This is training data that is used in the prediction of the prophages. If possible, you should use the") 
	print ("closest relative to your genome, however if that is not possible, you can also use the Generic Training set.")
	print ("\n")

######
def help_options():
	print ("\nUSAGE: python %s gbk_file outdir name trainingSet\n"  %os.path.realpath(__file__))
	
	print("Provide a single trainingSet number")
	print_list_modified()	


######
def main():
	## control if options provided or help
	if len(sys.argv) != 5:
		help_options()
		exit()
	
	## argv
	gbk_file = os.path.abspath(argv[1])
	outdir = os.path.abspath(argv[2])
	name=argv[3]
	training_set = argv[4]
	
	## call
	print ("\n+ Generate annotation of putative phages for sample %s in folder: %s" %(name, outdir))
	ident_bacteriophage(gbk_file=gbk_file, name=name, output_dir=outdir, training_set=training_set, Debug=False)
	results_PhiSpy(outdir, name)
	
	
	
######
'''******************************************'''
if __name__== "__main__":
	main()



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################

## useful imports
import time
import io
import os
import re
import sys
import concurrent.futures
from termcolor import colored
import pandas as pd
import shutil

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.scripts import spades_assembler
from BacterialTyper.scripts import annotation
from BacterialTyper.scripts import sampleParser
from BacterialTyper.scripts import BUSCO_caller
from BacterialTyper.modules import qc
from BacterialTyper.modules import help_info

##
global assembly_stats
assembly_stats = {}

####################################
def run_assembly(options):
	"""Main function of the assemble module.
	
	It assembles each sample using SPADES_ and checks quality using BUSCO_ software and database.

	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.sampleParser.help_format`
	
		- :func:`BacterialTyper.scripts.BUSCO_caller.print_help_BUSCO`
	
		- :func:`BacterialTyper.scripts.info.project_help`
		
		- :func:`BacterialTyper.scripts.multiQC_report.multiqc_help`
		
		- :func:`BacterialTyper.scripts.functions.pipeline_header`
		
		- :func:`BacterialTyper.scripts.functions.boxymcboxface`
		
		- :func:`BacterialTyper.scripts.functions.print_time`
		
		- :func:`BacterialTyper.scripts.sampleParser.get_files`
		
		- :func:`BacterialTyper.scripts.functions.outdir_project`
		
		- :func:`BacterialTyper.scripts.functions.optimize_threads`
		
		- :func:`BacterialTyper.modules.assemble.check_sample_assembly`
		
		- :func:`BacterialTyper.scripts.functions.create_subfolder`
		
		- :func:`BacterialTyper.modules.qc.BUSCO_check`
		
	.. include:: ../../links.inc	 	
	
	"""
	
	## init time
	start_time_total = time.time()
	
	## debugging messages
	global Debug
	if (options.debug):
		Debug = True
	else:
		Debug = False
		
	##################################
	### show help messages if desired	
	##################################
	if (options.help_format):
		## help_format option
		sampleParser.help_format()
		exit()
	elif (options.help_BUSCO):
		## information for BUSCO
		BUSCO_caller.print_help_BUSCO()
		exit()
	elif (options.help_project):
		## information for project
		help_info.project_help()
		exit()
	elif (options.help_multiqc):
		## information for Multiqc
		multiQC_report.multiqc_help()
		exit()
	
	### set as default paired_end mode
	if (options.single_end):
		options.pair = False
	else:
		options.pair = True

	## message header
	functions.pipeline_header()
	functions.boxymcboxface("Assembly module")
	print ("--------- Starting Process ---------")
	functions.print_time()
	
	## absolute path for in & out
	input_dir = os.path.abspath(options.input)
	outdir=""

	## Project mode as default
	project_mode=True
	if (options.detached):
		options.project = False
		project_mode=False
		outdir = os.path.abspath(options.output_folder)
	else:
		options.project = True
		outdir = input_dir	
		
	## get files
	pd_samples_retrieved = sampleParser.get_files(options, input_dir, "trim", ['_trim'])
	
	## debug message
	if (Debug):
		print (colored("**DEBUG: pd_samples_retrieve **", 'yellow'))
		print (pd_samples_retrieved)
	
	## generate output folder, if necessary
	print ("\n+ Create output folder(s):")
	if not options.project:
		functions.create_folder(outdir)
	outdir_dict = functions.outdir_project(outdir, options.project, pd_samples_retrieved, "assemble")

	### call assemble using spades
	start_time_partial = start_time_total
	start_time_partial_assembly = start_time_partial
	
	## optimize threads
	name_list = set(pd_samples_retrieved["name"].tolist())
	threads_job = functions.optimize_threads(options.threads, len(name_list)) ## threads optimization
	max_workers_int = int(options.threads/threads_job)

	## debug message
	if (Debug):
		print (colored("**DEBUG: options.threads " +  str(options.threads) + " **", 'yellow'))
		print (colored("**DEBUG: max_workers " +  str(max_workers_int) + " **", 'yellow'))
		print (colored("**DEBUG: cpu_here " +  str(threads_job) + " **", 'yellow'))

	# Group dataframe by sample name
	sample_frame = pd_samples_retrieved.groupby(["name"])

	# We can use a with statement to ensure threads are cleaned up promptly
	print ('+ Running modules SPADES...')
	with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers_int) as executor:
		## send for each sample
		commandsSent = { executor.submit( check_sample_assembly, name, outdir_dict[name],  sorted(cluster["sample"].tolist()), threads_job): name for name, cluster in sample_frame }

		for cmd2 in concurrent.futures.as_completed(commandsSent):
			details = commandsSent[cmd2]
			try:
				data = cmd2.result()
			except Exception as exc:
				print ('***ERROR:')
				print (cmd2)
				print('%r generated an exception: %s' % (details, exc))
		
	## functions.timestamp
	print ("\n+ Assembly of all samples finished: ")
	start_time_partial = functions.timestamp(start_time_partial_assembly)

	###################
	if Debug:
		print ("** DEBUG: assembly_stats dictionary")
		print (assembly_stats)
	
	##
	if (assembly_stats):
		## get all assembly stats
		outdir_report = functions.create_subfolder("report", outdir)
		final_dir = functions.create_subfolder("assembly_stats", outdir_report)
		final_sub_dir = functions.create_subfolder("samples", final_dir)
		

		#### summary and information
		results_summary_toPrint_all = pd.DataFrame()		
		column_names = ("Total Sequences", "Total Length (bp)", "Total length (no Ns)", 
                "Adenine (A)", "Thymine (T)", "Cytosine (C)", "Guanine (G)", "A+T", "C+G", "Any Nucleotide (N)", 
                "Capture Gaps", "Capture Gaps Length", "Capture Gaps Length/Total Length (%)", 
                "Number Seqs > 10 kb", "% Seqs > 10 kb", "Total Length (bp) > 10 kb", "% Bases > 10 kb", 
                "MinLen", "MaxLen", "Average Len", "Median Len", "N50", "L50", "Length (no Ns)")
		
		for stats in assembly_stats:
			##
			file_stats = assembly_stats[stats]
			shutil.copy(file_stats, final_sub_dir)
		
			tmp_name = os.path.splitext(file_stats)
			#csv
			csv_file = tmp_name[0] + '.csv'

			######
			if Debug:
				print ("file_stats: ", file_stats)
				print ("tmp_name: ", tmp_name)
				print ("csv_file: ", csv_file)

			## subset dataframe	& print result
			results_summary_toPrint = pd.read_csv(csv_file, sep=',', header=None).transpose()
			
			## TODO: check if file read contains the correct amount of columns.
			## It might produce stop failure if not matches.

			results_summary_toPrint.columns = column_names
			results_summary_toPrint['sample'] = stats
		
			## add all data
			results_summary_toPrint_all = pd.concat([results_summary_toPrint_all, results_summary_toPrint], ignore_index=True)

		## write to excel
		name_excel_summary = final_dir + '/summary_stats.xlsx'
		writer_summary = pd.ExcelWriter(name_excel_summary, engine='xlsxwriter') ## open excel handle
		
		## filter important columns		
		results_summary_toPrint_all = results_summary_toPrint_all.set_index('sample')			
		results_summary_toPrint_all = results_summary_toPrint_all[["Total Sequences", "Total Length (bp)", "Total length (no Ns)",
														"Adenine (A)", "Thymine (T)", "Cytosine (C)", "Guanine (G)", "A+T", "C+G", "Any Nucleotide (N)",
                                                  		"MinLen", "MaxLen", "Average Len", "Median Len", "N50", "L50"]]
		
		
		## >10 kb subset
		subset_summary_toPrint_all = results_summary_toPrint_all[["Total Sequences", "Total Length (bp)", "Median Len", "N50", "L50"]]
		subset_summary_toPrint_all.to_excel(writer_summary, sheet_name="summary") ## write excel handle
		
		## nucleotide tab
		nucleotides_summary_toPrint_all = results_summary_toPrint_all[["Adenine (A)", "Thymine (T)", "Cytosine (C)", "Guanine (G)", "A+T", "C+G", "Any Nucleotide (N)"]]
		nucleotides_summary_toPrint_all.to_excel(writer_summary, sheet_name="nucleotides") ## write excel handle
		
		results_summary_toPrint_all.to_excel(writer_summary, sheet_name="all_data") ## write excel handle
		
		writer_summary.save() ## close excel handle
	
	### symbolic links
	print ("+ Retrieve all genomes assembled...")
	
	### BUSCO check assembly
	if (options.no_BUSCO):
		print ()
	else:
		results = qc.BUSCO_check(outdir, outdir, options, start_time_partial, "genome")
	
	## print to file results	 	
	print ("\n*************** Finish *******************")
	start_time_partial = functions.timestamp(start_time_total)

	print ("+ Exiting Assembly module.")
	return()

#############################################
def check_sample_assembly(name, sample_folder, files, threads):
	"""Checks if sample is assembled.
	
	It checks whether a sample is assembled or not by reading file *sample_folder/.success_all* (generated by :func:`BacterialTyper.scripts.functions.print_time_stamp`). 
	
	If file not available (no previous assembly or not suceeded it) it calls :func:`BacterialTyper.scripts.spades_assembler.run_module_assembly` to generate assembly for the sample speficied.
	
	:param name: Sample name or tag to identify sample.
	:param sample_folder:  directory to generate assembly ouptut. It must exist.
	:param files: List containing files (fastq R1 & R2) for the sample to be assembled.
	:param threads: Number of CPUs to use
	:type name: string
	:type sample_folder: string 
	:type files: list
	:type threads: integer
	
	:return: Populates dictionary assembly_stats with assembly stats information file
	:rtype: Dataframe
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.spades_assembler.run_module_assembly`
	
		- :func:`BacterialTyper.scripts.functions.print_time_stamp`
	
	"""
	## check if previously assembled and succeeded
	filename_stamp = sample_folder + '/.success_all'
	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		print (colored("\tA previous command generated results on: %s [%s]" %(stamp, name), 'yellow'))
		assembly_stats[name] = sample_folder + '/' + name + "_assembly.fna_stats.txt"

	else:
	
		## debug message
		if (Debug):
			print (colored("**DEBUG: spades_assembler.run_module_assembly call**", 'yellow'))
			print ("spades_assembler.run_module_assembly(name, sample_folder, files[0], files[1], threads)")
			print ("spades_assembler.run_module_assembly " + name + "\t" + sample_folder + "\t" + files[0] + "\t" + files[1] + "\t" +str(threads) + "\n")
	
		# Call spades_assembler
		code = spades_assembler.run_module_assembly(name, sample_folder, files[0], files[1], threads)
		
		if (code != 'FAIL'):
			## success stamps
			filename_stamp = sample_folder + '/.success_all'
			stamp =	functions.print_time_stamp(filename_stamp)
			assembly_stats[name] = code




#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Generates a multi locus sequence typing (MLST) using MLSTar R package.
- Downloads profiles
- Searches assembly fasta sequences
- Plots MLST profiles
"""

## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
import pandas as pd
from termcolor import colored
import shutil

import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt
from pandas.plotting import table
from matplotlib.backends.backend_pdf import PdfPages

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config
from BacterialTyper.other_tools import tools
from BacterialTyper.data import data_files

## R scripts
MLSTarR_script = tools.R_scripts('MLSTar_call')
MLSTarR_plot = tools.R_scripts('MLSTar_plot')
MLSTarR_download_seq = tools.R_scripts('MLSTar_downloadPubMLST_seq')
MLSTarR_download_prf = tools.R_scripts('MLSTar_downloadPubMLST_profile')
MLSTarR_getpubmlst = tools.R_scripts('MLSTar_getpubmlst')

##########################################
def help_MLSTar():
	'''
	Provides help when script call as a single script.
	'''
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))
	get_MLSTar_species()
	print ("")

##########################################
def run_MLSTar(species_mlst_folder, rscript, species, scheme, name, path, fileGiven, threads):
	
	## fileGiven = fasta file

	## PubMLST folder
	#pubmlst_folder = functions.create_subfolder('PubMLST', database_folder)
	
	## species folder
	#species_mlst_folder = functions.create_subfolder(species, pubmlst_folder)
	
	## scheme folder
	scheme_name = 'scheme_' + str(scheme)
	scheme_folder = functions.create_subfolder(scheme_name, species_mlst_folder)

	## seq/profile folder
	seq_folder = functions.create_subfolder('seq', scheme_folder)
	profile_folder = functions.create_subfolder('prf', scheme_folder)

	print ("##################################################")
	print ("+ MLST profiling for sample: %s" %name)
	print ("##################################################")

	## check if profile and sequences are already downloaded
	download_PubMLST(profile_folder, scheme, seq_folder, rscript, species)

	## call MLSTar for this sample
	results = run_doMLST(profile_folder, seq_folder, name, rscript, path, fileGiven, threads)
	
	return (results, profile_folder)

##########################################
def run_doMLST(profile_folder, seq_folder, name, rscript, path, fileGiven, threads):

	print ('+ Generating profile for sample...')

	folder_results = os.path.join(path, name  + '_alleles') 
	
	## success timestamp
	filename_stamp = path + '/.success'
	if os.path.isfile(filename_stamp):
		stamp =	functions.read_time_stamp(filename_stamp)
		print (colored("\tA previous command generated results on: %s [%s]" %(stamp, name), 'yellow'))
		res_file = path + '/' + name + "_MLST_results.csv"
		return(res_file)
		
	else:
		if os.path.exists(folder_results):
			shutil.rmtree(folder_results)
			
		logFile = path + '_logFile.txt'
		cmd_profiler = "%s %s --dir_profile %s --dir_seq %s --file %s --dir %s --name %s --threads %s --lib.loc %s 2> %s" %(rscript, MLSTarR_script, profile_folder, seq_folder, fileGiven, path, name, threads, get_MLSTar_package_installed(), logFile)
		callCode = functions.system_call(cmd_profiler)

	if callCode == 'OK':
		res_file = path + '/' + name + "_MLST_results.csv"
		stamp =	functions.print_time_stamp(filename_stamp)
		return (res_file)
	else:
		return ('FAIL')

##########################################
def update_MLSTar_profile_alleles():
	return("")
	# [TODO: update_MLSTar_profile_alleles():
	## check folder *_alleles/ and add information *_MLST_results.csv
	
##########################################
def get_MLSTar_species(genus, species):

	"""
	Retrieve the correct name within `PubMLST databases`_ for the given species and/or genus specified.
	
	:param genus: Genus name.
	:param species: Species name.
	:type genus: string
	:type species: string

	:returns: Name in the PubMLST database for the taxa of interest. Returns `NaN` if not available. 
	
	Available datasets are stored in file :file:`/data/PubMLST_datasets.csv`. See available :doc:`PubMLST datasets<../../../data/PubMLST_datasets>`.
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.data.data_files.data_list`
	
	"""
	
	## MLSTar available data
	MLSTar_species = data_files.data_list("PubMLST_datasets")
	
	# pandas from csv file
	data = pd.read_csv(MLSTar_species, names=["Kingdom","name","Species"], sep=",")
	
	## check if name matches	
	taxa_name = genus + ' ' + species
	sp_exists = data.loc[data["Species"] == taxa_name]['name']
	if not sp_exists.empty: ## check if exists
		return (sp_exists.values[0])
	else:
		## TODO: Check if it works
		## Check if there is a genus entry in database
		genus_name = genus + ' spp.'
		genus_exist = data.loc[data["Species"] == genus_name]['name']

		if not genus_exist.empty: ## check if exists
			return (genus_exist.values[0])
			
	## we have a problem...
	## neither a taxa or genus exists...
	return ('NaN')
		
##########################################
def getPUBMLST(species, rscript, out_name):
	"""
	Using `MLSTar software`_ retrieve for the given `species` the available schemes in PubMLST_.  

	It generates information in file `out_name` in csv format. 
	
	See example in :file:`/devel/results/getPubMLST_example.csv`
	
	.. include:: ../../devel/results/getPubMLST_example.csv
		:literal:
	   
	:param species: Sample name or tag to identify sample
	:param rscript: Path to Rscript to generate system call.
	:param out_name: Output file to generate profile information.

	:type species: string
	:type rscript: string
	:type out_name: string

	:return: OK/FAIL
	:warnings: Returns **FAIL** if process stopped.
	
	.. include:: ../../links.inc

	"""
	
	MLSTarR_getpubmlst = tools.R_scripts('MLSTar_getpubmlst')

	## species is a comma separated string
	path_package = get_MLSTar_package_installed()
	#MLSTar_getpubmlst
	cmd_getPUBMLST = "%s %s --species %s --output %s --lib.loc %s 2> /dev/null" %(rscript, MLSTarR_getpubmlst, species, out_name, path_package)
	return(functions.system_call(cmd_getPUBMLST))

##########################################
def plot_MLST(results, profile, rscript):
	path_package = get_MLSTar_package_installed()
	path_folder = os.path.dirname(results)
	cmd_plotter = "%s %s --output %s --folder_profile %s --file_result %s --lib.loc %s" %(rscript, MLSTarR_plot, path_folder, profile, results, path_package)
	return(functions.system_call(cmd_plotter))
	
##########################################
def download_PubMLST(profile_folder, scheme, seq_folder, rscript, species):

	## Check if profile exists
	file_prof = profile_folder + '/profile_scheme' + str(scheme) + '.tab'

	######################
	## download profile ##
	######################
	if os.path.exists(file_prof):
		print ('+ Profile file for scheme exists...')

		## check if previously download profile and succeeded
		filename_stamp = profile_folder + '/.success'

		if os.path.isfile(filename_stamp):
			stamp =	functions.read_time_stamp(filename_stamp)
			print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, species, 'profile'), 'yellow'))
		#
		#else
		# [TODO: Check time passed and download again if >?? days passed]
		
	else:
		print ('+ Profile file for scheme will be downloaded...')
		print ('+ Downloading profile...')
		logFile = profile_folder + '_download.log'
		cmd_profile = "%s %s --species %s --scheme %s --dir_profile %s --lib.loc %s 2> %s" %(rscript, MLSTarR_download_prf, species, scheme, profile_folder, get_MLSTar_package_installed(), logFile)
		callCode = functions.system_call(cmd_profile)
		
		if (callCode == 'OK'):
			## success timestamp
			filename_stamp = profile_folder + '/.success'
			stamp =	functions.print_time_stamp(filename_stamp)
	
	#######################
	## download sequence ##
	#######################
	seq_bool = 0
	if os.path.exists(seq_folder):
		print ('+ Sequence folder exists...')
		
		## check if previously download sequence and succeeded
		filename_stamp = seq_folder + '/.success'
		
		if os.path.isfile(filename_stamp):
			stamp =	functions.read_time_stamp(filename_stamp)
			print (colored("\tA previous command generated results on: %s [%s -- %s]" %(stamp, species,'sequence'), 'yellow'))

			########################################################################
			#else
			# [TODO: Check time passed and download again if >?? days passed]
			########################################################################
			#files = os.listdir(seq_folder)
			#count_fas = 0		
			#for f in files:
			#	if f.endswith('.fas'):
			#		count_fas += 1
			#	else:				
			#		os.remove(seq_folder + '/' + f)
			#
			#if count_fas > 6:
			#	print ("+ Assuming sequences are previously downloaded...")
			#else:
			#	seq_bool = 1		
			#	os.rmdir(seq_folder)
			#	seq_folder_path = functions.create_folder(seq_folder)
			
		else:
			print ('+ Sequence files for scheme will be downloaded...')
			print ('+ Downloading sequences...')
			logFile = seq_folder + '_download.log'
			cmd_seq = "%s %s --species %s --scheme %s --dir_seq %s --lib.loc %s 2> %s" %(rscript, MLSTarR_download_seq, species, scheme, seq_folder, get_MLSTar_package_installed(), logFile)
			callCode = functions.system_call(cmd_seq)
			
			if (callCode == 'OK'):
				## success timestamp
				filename_stamp = seq_folder + '/.success'
				stamp =	functions.print_time_stamp(filename_stamp)

def get_MLSTar_package_installed():
	return (set_config.R_package_path_installed())
	

##########################################
def main():
	## this code runs when call as a single script

  	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()
	
	#fileGiven = os.path.abspath(argv[1])
	genome = os.path.abspath(argv[1]) ## assembly
	species = argv[2]
	rscript = argv[3]
	database = os.path.abspath(argv[4])
	name = argv[5]
	path = os.path.abspath(argv[6])
	
	##
	threads = 1
	functions.create_folder(path)
	
	## PubMLST folder
	pubmlst_folder = functions.create_subfolder('PubMLST', database)
	
	## species folder
	species_mlst_folder = functions.create_subfolder(species, pubmlst_folder)

	## output file
	output_file = species_mlst_folder + '/PubMLST_available_scheme.csv'
	
	### get scheme available
	getPUBMLST(species, rscript, output_file)
	
	## parse PubMLST results	
	scheme = 1
		
	## call MLST
	(results, profile_folder) = run_MLSTar(species_mlst_folder, rscript, species, scheme, name, path, genome, threads)
		
	## plot MLST
	plot_MLST(results, profile_folder, rscript)

##########################################
def help_options():
	print ("\nUSAGE: python %s genome species rscript_bin database name path\n"  %os.path.realpath(__file__))

		
##########################################
'''******************************************'''
if __name__== "__main__":
	main()


#	fileName = "MLSTar_species"
#	data.to_csv(fileName + '.txt', sep='\t')
#
#	## plot table
#	pdf_name = fileName + '.pdf'
#	pp = PdfPages(pdf_name)
#
#	fig,ax = plt.subplots()
#	fig.patch.set_visible(False)
#	
#	ax.axis('off')
#	ax.axis('tight')
#	
#	## color according to kingdom
#	colors = data.applymap(lambda x: 'palegreen' if x== 'bacteria' else ('lightyellow' if x== 'eukarya' else ('lightsalmon' if x=='other' else 'white' )))
#	## https://www.rapidtables.com/web/color/html-color-codes.html
#	
#	tab = ax.table(
#		cellText=data.values,
#		colLabels=data.columns,
#		cellColours =colors.values,
#		colWidths=[0.26 for x in data.columns],
#		loc='center', colLoc = 'center', rowLoc='left', cellLoc='center')
#	tab.auto_set_font_size(False)
#	tab.set_fontsize(5)
#
#	fig.tight_layout()	
#	pp.savefig()
#	plt.close()
#	pp.close()	



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez                                      ##
## Copyright (C) 2019-2020 Lauro Sumoy Lab, IGTP, Spain ##
##########################################################
"""
Retrieves genes ids from profile analysis generated
"""
## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
from termcolor import colored
import pandas as pd
from Bio import SeqIO

## import my modules
from BacterialTyper.config import set_config

## import my HCGB module 
from HCGB.functions import main_functions

##############
def help_options():
    print ("\nUSAGE: python %s gene_id profile_folder...\n"  %os.path.realpath(__file__))

##############
def help_retrieve_genes():
    print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))

##############
def retrieve_genes_ids_sequences(profile, gene_ID, debug):
    """    
    """
    ## given a profile folder
    if debug:
        print ("** DEBUG **")
        print ("profile: ", profile)
        print ("gene_id: ", gene_ID)
        
    ##
    assembled_genes_list = main_functions.retrieve_matching_files(profile, "assembled_genes.fa", debug)
    assembled_genes_list = [s for s in assembled_genes_list if 'ariba.tmp' not in s]
    
    if debug:
        print ("** DEBUG **")
        print ("assembled_genes_list: ", assembled_genes_list)

    if os.path.isfile(assembled_genes_list[0]):
        for record in SeqIO.parse(assembled_genes_list[0], "fasta"):
            if debug:
                print ("** DEBUG: ", record.description, " **")
                #print (record.seq)
 
            search_ID = re.search(gene_ID, record.description)
            if (search_ID):
                return (record.id, str(record.seq))

        return('','')

##############
def get_genes_profile(samples_info, gene_names, debug, option):
    """    
    """
    ## search by group id or gene name
    print ('\n+ Retrieve selected genes profile for each sample.')
    results_profileIDs = pd.DataFrame()
    sample_frame = samples_info.groupby(["name"])
    for g in gene_names:
        #print ("\t+", g)
        for name, cluster_df in sample_frame:
            my_list_profiles = cluster_df.loc[cluster_df['tag'] == 'profile']['ext'].to_list()
	   
            if debug:
                print ("name: ", name)
                print ("my_list_profiles:")
                print (my_list_profiles)
                print ("cluster_df")
                print (cluster_df)

            fill=False
            for p in my_list_profiles:
                profile_csv = cluster_df.loc[cluster_df['ext'] == p]['sample'].to_list()[0]
                if debug:
                    print ("profile_csv: ", profile_csv)
            
                value = retrieve_genes_ids_profile(profile_csv, g, debug, option)
            
                ## save results 
                if (not value.empty):
                    for Name, Data in value.iterrows():
                        results_profileIDs.loc[name,Name] = Data['Status']
                    fill=True

            if not fill:
                results_profileIDs.loc[name, g] = 'no'

    return (results_profileIDs)

##############
def retrieve_genes_ids_profile(profile, gene_ID, debug, option):
    """    
    """
    ## read data    
    get_csv_data = functions.main_functions.get_data(profile, ',', '')
    
    if option == 'name':
        list_Genes = get_csv_data['Genes'].to_list()
        get_csv_data.index = get_csv_data['Genes']
    elif option == 'ID':
        list_Genes = get_csv_data['ID'].to_list()
        get_csv_data.index = get_csv_data['ID']

    
    ## debug messages
    if debug:
        print ("** DEBUG **")
        print ("profile: ", profile)
        print ("gene_id: ", gene_ID)
        print ("data")
        print (get_csv_data)
        print ("Option: ", option)
        print ("Genes")
        print (list_Genes)
        
    ## search accordingly
    if option == 'name':
        regex_search = re.compile("^" + gene_ID + ".*")
        filtered_genes = list(filter(regex_search.match, list_Genes))
        
        ## debug messages
        if debug:
            print ("** DEBUG **")
            print (filtered_genes)
            print (get_csv_data.loc[filtered_genes])
        
        return (get_csv_data.loc[filtered_genes]) 
        
    else:
        if gene_ID in list_Genes:
            ## debug messages
            if debug:
                print ("** DEBUG **")
                print ("gene_ID: ", gene_ID)
                print (get_csv_data.loc[gene_ID])
            return (get_csv_data.loc[gene_ID])
        else:
            return(pd.DataFrame()) 
    
##############
def main():

    ## control if options provided or help
    if len(sys.argv) > 1:
        print ("")
    else:
        help_options()
        exit()

    ##
    gene_ID = sys.argv[1]
    profile = os.path.abspath( sys.argv[2] )
    
    retrieve_genes_ids_profile(profile, gene_ID, True)

'''******************************************'''
if __name__== "__main__":
    main()



#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
It generates a protein annotation using Prokka_ for the genomes assembled generated

Prokka_ contains several databases with known parameters for several kingdom and genus. Prokka option `--usegenus` is set as default.
Several options are available for kingdom and genus. See details below and see output from :code:`prokka --listdb`
shown in :func:`BacterialTyper.scripts.annotation.print_list_prokka`. If no genus matches user desired option, use option "Other".

.. include:: ../../links.inc	 	
"""

## useful imports
import time
import io
import os
import re
import sys
from sys import argv
from io import open
import pandas as pd
from termcolor import colored

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

### print info prokka
def print_list_prokka():
	"""
	Prints Prokka_ databases that has installed to use. It is the output from the call: 
	
	.. code-block:: sh

		prokka --listdb
	
	.. include:: ../../devel/results/print_list_prokka.txt
		:literal:
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.set_config.get_exe`
		
		- :func:`BacterialTyper.scripts.functions.system_call`
	
	.. include:: ../../links.inc	 	
	"""
	prokka_bin = set_config.get_exe('prokka')
	cmd = prokka_bin + " --listdb"
	functions.system_call(cmd)

######
def module_call(sequence_fasta, kingdom, genus, path, name, threads):
	"""
	Function that checks and generates annotation.
	
	- It uses Prokka_ via :func:`BacterialTyper.scripts.annotation.prokka_call`.
	
	- It checks if previously generated using :func:`BacterialTyper.scripts.functions.read_time_stamp`. 
	
	- Once finished, it prints timestamp (:func:`BacterialTyper.scripts.functions.print_time_stamp`). 
	
	:param sequence_fasta: Assembled sequences in fasta file format. 
	:param kingdom: Available kingdoms mode for Prokka software: Archaea|Bacteria|Mitochondria|Viruses
	:param genus: Available genus options for Prokka software. See details above.
	:param path: Absolute path to the output folder to include results.
	:param name: Sample name and tag to include in the annotation report and files.
	:param threads: Number of CPUs to use.
	  
	:type sequence_fasta: string
	:type kingdom: string
	:type genus: string 
	:type path: string 
	:type name: string 
	:type threads: integer 
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.set_config.get_exe`
		
		- :func:`BacterialTyper.scripts.functions.read_time_stamp`
		
		- :func:`BacterialTyper.scripts.functions.print_time_stamp`
				
		- :func:`BacterialTyper.scripts.annotation.prokka_call`	

	.. include:: ../../links.inc	 	
	"""
	
	## check if previously assembled and succeeded
	filename_stamp = path + '/.success'

	if os.path.isdir(path):
		if os.path.isfile(filename_stamp):
			stamp =	functions.read_time_stamp(filename_stamp)
			print (colored("\tA previous command generated results on: %s [%s]" %(stamp, name), 'yellow'))
			return ()
	
	## call prokka
	prokka_bin = set_config.get_exe('prokka')
	dirname = prokka_call(prokka_bin, sequence_fasta, kingdom, genus, path, name, threads)

	## success stamps
	filename_stamp = path + '/.success'
	stamp =	functions.print_time_stamp(filename_stamp)

	return(dirname)	

######
def prokka_call(prokka_bin, sequence_fasta, kingdom, genus, outdir_name, name, threads):
	"""Create system call for Prokka_ software. 
	
	It generates genome annotation using Prokka software. 
		
	:param prokka_bin: Path to the prokka binary file.
	:param sequence_fasta: Assembled sequences in fasta file format. 
	:param kingdom: Available kingdoms mode for Prokka software: Archaea|Bacteria|Mitochondria|Viruses
	:param genus: Available genus options for Prokka software. See details above.
	:param outdir_name: Absolute path to the output folder to include results.
	:param name: Sample name and tag to include in the annotation report and files.
	:param threads: Number of CPUs to use.
	  
	:type prokka_bin: string
	:type sequence_fasta: string
	:type kingdom: string
	:type genus: string 
	:type outdir_name: string 
	:type name: string 
	:type threads: integer 
	
	.. seealso:: Check description of output files generated in:
	
		- :ref:`Prokka-output-files`


	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.set_config.get_exe`
		
		- :func:`BacterialTyper.scripts.functions.read_time_stamp`
		
		- :func:`BacterialTyper.scripts.functions.print_time_stamp`
				
		- :func:`BacterialTyper.scripts.annotation.prokka_call`	
	
	.. include:: ../../links.inc
	"""
	
	## set parameters and options for prokka
	print ("\n+ Starting annotation for: %s\n" %name)
	log_file = outdir_name + '/run.log'
	options = "--cdsrnaolap --addgenes --addmrna --kingdom " + kingdom
	if genus != "Other":
		options = options + " --usegenus --genus " + genus
	prokka = "%s --force --outdir %s --prefix %s --locustag %s %s --cpus %s %s 2> %s" %(prokka_bin, outdir_name, name, name, options, threads, sequence_fasta, log_file)
	functions.system_call(prokka)
	return(outdir_name)

######
def help_options():
	print ("\nUSAGE: python %s sequence_file path name CPUs kingdom prokka_bin\n"  %os.path.realpath(__file__))

######
def main():

 	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()
	
	## argv
	seq_file = os.path.abspath(argv[1])
	folder = os.path.abspath(argv[2])
	name = argv[3]
	threads = int(argv[4])
	kingdom = argv[5]
	prokka_bin = argv[6]
	
	## call
	dir_annot=prokka_call(prokka_bin, seq_file, kingdom, folder, name, threads)
	print ("\n+ Annotation for sample %s has been generated in folder: %s" %(name, dir_annot))

######
'''******************************************'''
if __name__== "__main__":
	main()



from pkg_resources import get_distribution
try:
	__version__ = get_distribution('BacterialTyper').version
except:
	try:
		__version__ = pkg_resources.resorce_filename('BacterialTyper', VERSION)
	except:
		__version__ = 'local'

### to include when distribution available
## version will be retrieve from setup.py

__all__ = [
	'data',
	'modules',
	'other_tools',
	'config',
	'scripts',
	'report'
]

from BacterialTyper import *







#!/usr/bin/env python3
##########################################################
## Jose F. Sanchez										##
## Copyright (C) 2019 Lauro Sumoy Lab, IGTP, Spain		##
##########################################################
"""
Calls multiQC_ to generate HTML statistics reports.

.. include:: ../../links.inc 
"""
## useful imports
import os
import io
import sys
from io import open
from sys import argv
from termcolor import colored

## import my modules
from BacterialTyper.scripts import functions
from BacterialTyper.config import set_config

############
def multiQC_module_call(givenList, name, path, option):
	"""
	Prepares files for multiQC report generation.
	
	:param givenList: List of folder to search for multiQC report.
	:param name: Name to include in the html report.
	:param path: Absolute path for the output folder.
	:param option: Some options to provide to multiQC_call.
	
	:type givenList: list
	:type name: string
	:type path: string
	:type option: string
	
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.printList2file`
		
		- :func:`BacterialTyper.scripts.multiQC_report.multiQC_call`
	
	"""
	pathFile = path + '/' + 'samples.txt'
	functions.printList2file(pathFile, givenList)
	multiQC_call(pathFile, name, path, option)	
	
############
def multiQC_call(pathFile, name, folder, option):
	"""
	multiQC_ report generation call.
	
	:param pathFile: File containing list of files to include in report.
	:param name: Name to include in the html report.
	:param folder: Absolute path for the output folder.
	:param option: Options to provide to multiQC call.
	
	:type pathFile: string
	:type name: string 
	:type folder: string 
	:type option: string
	
	:returns: :func:`BacterialTyper.scripts.functions.system_call` output (OK/FALSE)
		
	.. seealso:: This function depends on other BacterialTyper functions called:
	
		- :func:`BacterialTyper.scripts.functions.system_call`
	
	"""
	multiqc_bin = "multiqc" ## if we activate the environment it should be in $PATH
	## set options for call
	cmd = "%s --force -o %s -n %s -l %s -p -i 'MultiQC report' -b 'HTML report generated for multiple samples and steps' %s" %(multiqc_bin, folder, name, pathFile, option)
	
	## if a report was previously generated in the folder 
	## force to delete and generate a new one
	return(functions.system_call(cmd))

############
def	help_options():
	"""
	multiQC_ help options for call as a single script.
	"""
	print ("\nUSAGE:\npython %s folder sample_list name\n"  %os.path.abspath(argv[0]))

############
def multiqc_help():
	"""
	multiQC_ software description message.
	"""
	## [TODO]
	print (colored("\n\n***** TODO: Generate this help message *****\n\n", 'red'))

############
def main():
  	
  	## control if options provided or help
	if len(sys.argv) > 1:
		print ("")
	else:
		help_options()
		exit()    	

	## ARGV
	folder_name = os.path.abspath(argv[1])
	samples_path = os.path.abspath(argv[2])
	name = argv[3]

	## call
	multiQC_call(samples_path, name, folder_name,"")
		
############
'''******************************************'''
if __name__== "__main__":
	main()
	
##Options multiqc, version 1.7 :
##  -f, --force                     Overwrite any existing reports
##  -d, --dirs                      Prepend directory to sample names
##  -dd, --dirs-depth INTEGER       Prepend [INT] directories to sample names. Negative number to take from start of path.
##  -s, --fullnames                 Do not clean the sample names (leave as full file name)
##  -i, --title TEXT                Report title. Printed as page header, used for filename if not otherwise specified.
##  -b, --comment TEXT              Custom comment, will be printed at the top of the report.
##  -n, --filename TEXT             Report filename. Use 'stdout' to print to standard out.
##  -o, --outdir TEXT               Create report in the specified output directory.
##  -t, --template [simple|default_dev|default|sections|geo] Report template to use.
##  --tag TEXT                      Use only modules which tagged with this keyword, eg. RNA
##  --view-tags, --view_tags        View the available tags and which modules they load
##  -x, --ignore TEXT               Ignore analysis files (glob expression)
##  --ignore-samples TEXT           Ignore sample names (glob expression)
##  --ignore-symlinks               Ignore symlinked directories and files
##  --sample-names PATH             File containing alternative sample names
##  -l, --file-list                 Supply a file containing a list of file paths to be searched, one per row
##  -e, --exclude [module name]     Do not use this module. Can specify multiple times.
##  -m, --module [module name]      Use only this module. Can specify multiple times.
##  --data-dir                      Force the parsed data directory to be created.
##  --no-data-dir                   Prevent the parsed data directory from being created.
##  -k, --data-format [json|yaml|tsv] Output parsed data in a different format. Default: tsv
##  -z, --zip-data-dir              Compress the data directory.
##  -p, --export                    Export plots as static images in addition to the report
##  -fp, --flat                     Use only flat plots (static images)
##  -ip, --interactive              Use only interactive plots (HighCharts Javascript)
##  --lint                          Use strict linting (validation) to help code development
##  --pdf                           Creates PDF report with 'simple' template. Requires Pandoc to be installed.
##  --no-megaqc-upload              Don't upload generated report to MegaQC, even if MegaQC options are found
##  -c, --config PATH               Specific config file to load, after those in MultiQC dir / home dir / working dir.
##  --cl-config, --cl_config TEXT   Specify MultiQC config YAML on the command line
##  -v, --verbose                   Increase output verbosity.
##  -q, --quiet                     Only show log warnings
##  --version                       Show the version and exit.
##  -h, --help                      Show this message and exit.

